# File: Cart.py
"""\
Cart

"""
# Copyright (c) 2005 by Weimar Desenvolvimento e Consultoria em Informatica
# Ltda (Brasil), jens quadrat - AGENTUR FUER NEUE MEDIEN OEG (Austria),
# BlueDynamics KEG (Austria), Zettai.net LCC (United States)
# Generator: ArchGenXML Version 1.4 devel 2 http://sf.net/projects/archetypes/
#
# GNU General Public Licence (GPL)
# 
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA  02111-1307  USA
#
__author__  = '''Fabiano Weimar dos Santos <xiru@xiru.org>, Jens Klein
<jensens@jensquadrat.de>, Phil Auersperg <phil@bluedynamics.org>'''
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *


from Products.PloneMallRegistry.ICart import ICart



from Products.PloneMallCart.config import *

##code-section module-header #fill in your manual code here
from Products.PloneMallRegistry.util import filterContents
from config import *
from Products.PloneMallRegistry.MallRegistry import FactoryRegistry
from AccessControl.unauthorized import Unauthorized
from Products.PloneMallRegistry.ICartItem import ICartItem
from Products.PloneMallRegistry.Currency import Currency
from Products.CMFCore.utils import getToolByName
from Products.CMFDefault.SkinnedFolder  import SkinnedFolder

##/code-section module-header

schema= Schema((
),
)


##code-section after-schema #fill in your manual code here
from Products.PloneMallRegistry.AllowedTypesByIface import AllowedTypesByIface
from OFS.ObjectManager import ObjectManager
##/code-section after-schema

class Cart(AllowedTypesByIface,BaseFolder):
    """
    Cart
    """
    security = ClassSecurityInfo()
    __implements__ = (getattr(AllowedTypesByIface,'__implements__',()),) + (getattr(BaseFolder,'__implements__',()),) + (ICart,)


    # This name appears in the 'add' box
    archetype_name             = 'Cart'

    portal_type = meta_type    = 'Cart' 
    allowed_content_types      = ['CartItem'] 
    allowed_content_interfaces = ['ICartItem'] 
    filter_content_types       = 1
    global_allow               = 0
    allow_discussion           = 0
    #content_icon               = 'Cart.gif'
    immediate_view             = 'base_view'
    default_view               = 'base_view'
    typeDescription            = "Cart"
    typeDescMsgId              = 'description_edit_cart'
    hasOrderShipping           = False

    schema = BaseFolderSchema + \
             schema

    ##code-section class-header #fill in your manual code here
    ##/code-section class-header

    # catalog methods
    def indexObject(self, *args, **kwargs): pass
    def unindexObject(self, *args, **kwargs): pass
    def reindexObject(self, *args, **kwargs): pass

    #Methods
    #methods from Interface ICart

#    def __init__():
#        self.hasOrderShipping = False
    
    
    security.declareProtected("View", 'getContents')
    def getContents(self, filter = {}):
        '''
        Return a list of objects that are inside the cart. The result
        could be filtered using a dictionary, where the keys are object
        properties and the values are acceptable values for these keys.
        '''

        return filterContents(self.objectValues(['CartItem']), filter)



    security.declareProtected("Buy item", 'addItem')
    def addItem(self, item, quantity=1):
        '''
        Add item to cart.
        '''

        self._checkSessionId()

        ci = None
        l_contents = self.getContents()
        for obj in l_contents:
            if obj.getMallItem() == item:
                ci = obj
                break
        if ci is None:
            cif = FactoryRegistry.getMallFactory('CartItem')
            id = str(len(l_contents) + 1)
            ci = cif(id)
            self._setObject(id, ci)
            ci = getattr(self, id)
            ci.setMallItem(item)
            ci.setQuantity(quantity)
            ci.setPrice(item.getPrice().getValue())
            ci.setCurrency(item.getPrice().getCurrencySymbol())
        else:
            # For now, when we add the same item to the cart more than
            # one time, we will add 1 to the item quantity.  In a
            # future implementation, where we would be worried about
            # different shipping for each cart item, this method
            # should be modified to not add to the quantity, but
            # allways add a new cart item.
            ci.setQuantity(ci.getQuantity() + quantity)



    security.declareProtected("Buy item", 'updateItem')
    def updateItem(self, cart_item, quantity):
        '''
        Update item quantity in the cart.
        '''

        self._checkSessionId()

        if not FactoryRegistry.objectSupportsInterface(cart_item, ICartItem):
            raise TypeError("cart item %s is not registered as a ICartItem implementation" % \
                cart_item.__class__.__name__)

        if quantity < 0:
            raise ValueError("'%d' is not a valid quantity." % quantity)

        if quantity == 0:
            self.delItem([cart_item.getId()])
        else:
            cart_item.setQuantity(quantity)



    security.declareProtected("Buy item", 'delItem')
    def delItem(self, cart_item):
        '''
        Delete item from the cart.
        '''

        self._checkSessionId()

#        self.manage_delObjects(cart_item)
        # hack to circumvent Unauthorized errors on cart delete
        ObjectManager.manage_delObjects(self, cart_item)

        # keep the cart items ids as a 'well formed' sequence
        old_ids = [obj.getId() for obj in self.getContents()]
        new_ids = [str(n + 1) for n in range(len(old_ids))]
        try:
            if old_ids != new_ids:
                self.manage_renameObjects(old_ids, new_ids)
        except Unauthorized:
            #the resequencing int really necessary, so we can ignore this exception
            pass



    security.declareProtected("View", 'getTotal')
    def getTotal(self):
        '''
        Return the cart total.
        '''
        ret = Currency()
        for obj in self.getContents():
            ret += obj.getTotal()
        return ret

    security.declareProtected("View", 'getValueTotal')
    def getValueTotal(self):
        '''
        Return the cart total.
        '''
        ret = 0.0
        for obj in self.getContents():
            ret += float(obj.getTotal().getValue())
        return ret             

    #manually created methods
    
    def getHandling(self):
        '''
        Return the handling total from the cart
        '''
        ret = Currency()
        
        for obj in self.getContents():
            ret += obj.getHandling()
            if self.hasOrderShipping != 1:
                if obj.getHandling() > 0:
                    self.hasOrderShipping = False
                else:
                    self.hasOrderShipping = True
            
        return ret

    def getWeight(self):
        '''
        Return the handling total from the cart
        '''
        ret = 0.0
        
        for obj in self.getContents():
            mall_item = obj.getMallItem()
            if hasattr(mall_item, 'Weight'):
                quantity = obj.getQuantity()
                total_weight = quantity * float(mall_item.Weight())
                ret += total_weight
        
        return ret

    def getPackageCount(self):
        '''
        Return the number of packages in the cart
        '''
        ret = 0
        
        for obj in self.getContents():
            mall_item = obj.getMallItem()
            if hasattr(mall_item, 'Weight'):
                quantity = obj.getQuantity()
                ret += int(quantity)
        
        return ret

    def _checkSessionId(self):

        # For security reasons, enforce that the REQUEST comes from
        # the same user that created the Cart. It is important to
        # avoid that other Anonymous user or a Member add items into
        # Cart of others.

        sid = self.REQUEST.SESSION.getId()
        cid = self.getId()
        if sid != cid:
            raise Unauthorized



registerType(Cart,PROJECTNAME)
# end of class Cart

##code-section module-footer #fill in your manual code here
if register_cart:
    FactoryRegistry.registerMallFactory('Cart', Cart, (ICart,))
##/code-section module-footer



