# File: CartItem.py
"""\
Cart Item

"""
# Copyright (c) 2005 by Weimar Desenvolvimento e Consultoria em Informatica
# Ltda (Brasil), jens quadrat - AGENTUR FUER NEUE MEDIEN OEG (Austria),
# BlueDynamics KEG (Austria), Zettai.net LCC (United States)
# Generator: ArchGenXML Version 1.4 devel 2 http://sf.net/projects/archetypes/
#
# GNU General Public Licence (GPL)
# 
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA  02111-1307  USA
#
__author__  = '''Fabiano Weimar dos Santos <xiru@xiru.org>, Jens Klein
<jensens@jensquadrat.de>, Phil Auersperg <phil@bluedynamics.org>'''
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *


from Products.PloneMallRegistry.ICartItem import ICartItem



from Products.PloneMallCart.config import *

##code-section module-header #fill in your manual code here
from Products.PloneMallRegistry.Currency import Currency, VOC_CURRENCY
from config import *
from Products.PloneMallRegistry.MallRegistry import FactoryRegistry
from Products.PloneMallRegistry.IMallItem import IMallItem
from Products.CMFCore.utils import getToolByName
##/code-section module-header

schema= Schema((
    ReferenceField('item',
        widget=ReferenceWidget(
            label='Item',
            label_msgid='PloneMallCart_label_item',
            description='Enter a value for item.',
            description_msgid='PloneMallCart_help_item',
            i18n_domain='plonemallcart',
        ),
        allowed_types=('SimpleItem','ItemVariant'),
        relationship="item",
        required=1,
    ),
    
    StringField('currency',
        widget=SelectionWidget
        (
            label='Currency',
            label_msgid='PloneMallCart_label_currency',
            description='Enter a value for currency.',
            description_msgid='PloneMallCart_help_currency',
            i18n_domain='plonemallcart',
        ),
        required=1,
        vocabulary=VOC_CURRENCY,
        default_method="getDefaultCurrency",
    ),
    
    FixedPointField('item_price',
        widget=DecimalWidget(
            label="Price",
            label_msgid='PloneMallCart_label_item_price',
            description='Enter a value for Price.',
            description_msgid='PloneMallCart_help_item_price',
            i18n_domain='plonemallcart',
        ),
        required=1,
    ),
    
    FixedPointField('item_quantity',
        widget=DecimalWidget(
            label="Quantity",
            label_msgid='PloneMallCart_label_item_quantity',
            description='Enter a value for Quantity.',
            description_msgid='PloneMallCart_help_item_quantity',
            i18n_domain='plonemallcart',
        ),
        required=1,
    ),
    
),
)


##code-section after-schema #fill in your manual code here
##/code-section after-schema

class CartItem(BaseFolder):
    """
    Cart Item
    """
    security = ClassSecurityInfo()
    __implements__ = (getattr(BaseFolder,'__implements__',()),) + (ICartItem,)


    # This name appears in the 'add' box
    archetype_name             = 'CartItem'

    portal_type = meta_type    = 'CartItem' 
    allowed_content_types      = [] 
    filter_content_types       = 0
    global_allow               = 0
    allow_discussion           = 0
    #content_icon               = 'CartItem.gif'
    immediate_view             = 'base_view'
    default_view               = 'base_view'
    typeDescription            = "CartItem"
    typeDescMsgId              = 'description_edit_cartitem'

    schema = BaseSchema + \
             schema

    ##code-section class-header #fill in your manual code here
    ##/code-section class-header

    # catalog methods
    def indexObject(self, *args, **kwargs): pass
    def unindexObject(self, *args, **kwargs): pass
    def reindexObject(self, *args, **kwargs): pass

    #Methods
    #methods from Interface ICartItem

    security.declareProtected("View", 'getMallItem')
    def getMallItem(self):
        '''
        Return the product.
        '''

        try:
            return self.getRefs('item')[0]
        except IndexError:
            return None



    security.declareProtected("Modify portal content", 'setMallItem')
    def setMallItem(self, item):
        '''
        Set the product.
        '''

        if not FactoryRegistry.objectSupportsInterface(item,IMallItem):
            raise TypeError("item %s is not registered as a IMallItem implementation" % \
                item.__class__.__name__)

        self.deleteReferences('item')
        self.addReference(item, 'item')



    security.declareProtected("View", 'getPrice')
    def getPrice(self):
        '''
        Return the product price.
        '''
        
        return Currency(self.getCurrency(), self.getItem_price())



    security.declareProtected("Modify portal content", 'setPrice')
    def setPrice(self,price):
        '''
        Set the product price.
        '''
        
        ## test if price is something like 'US$ 10.99'
        l = price.split(' ')
        if len(l) == 2:
            self.setCurrency(l[0])
            self.setItem_price(l[1])
        else:
            self.setItem_price(l[0])



    security.declareProtected("View", 'getQuantity')
    def getQuantity(self):
        '''
        Return the product quantity.
        '''
        
        return float(self.getItem_quantity())



    security.declareProtected("Modify portal content", 'setQuantity')
    def setQuantity(self,quantity):
        '''
        Set the product quantity.
        '''
        
        self.setItem_quantity(str(quantity))



    security.declareProtected("View", 'getTotal')
    def getTotal(self):
        '''
        Return the cart item total.
        '''
        
        return self.getPrice() * self.getQuantity()


    #manually created methods

    def getDefaultCurrency(self):
        return self.portal_mall_registry.getDefaultCurrency()



registerType(CartItem,PROJECTNAME)
# end of class CartItem

##code-section module-footer #fill in your manual code here
if register_cart:
   FactoryRegistry.registerMallFactory('CartItem', CartItem, (ICartItem,))
##/code-section module-footer



