# File: AVS.py
# 
# Copyright (c) 2005 by Weimar Desenvolvimento e Consultoria em Informatica
# Ltda (Brasil), jens quadrat - AGENTUR FUER NEUE MEDIEN OEG (Austria),
# BlueDynamics KEG (Austria), Zettai.net LCC (United States)
# Generator: ArchGenXML Version 1.4.0-beta1 http://sf.net/projects/archetypes/
#
# GNU General Public Licence (GPL)
# 
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA  02111-1307  USA
#
__author__  = ''' <>, Fabiano Weimar dos Santos <xiru@xiru.org>, Jens Klein
<jensens@jensquadrat.de>, Phil Auersperg <phil@bluedynamics.org>'''
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *

from Products.PloneMallRegistry.IAVS import IAVS


from Products.PloneMallCustomer.config import *

from Products.CMFCore.utils import UniqueObject

    
##code-section module-header #fill in your manual code here
from config import *
from Products.PloneMallRegistry.MallRegistry import FactoryRegistry
from Products.PloneMallRegistry.Widget import LargeMultiSelectionWidget

from ISO3166Parser import ISO3166Parser
from Globals import package_home
from Products.PloneMallCustomer import product_globals
from os.path import join

iso3166_parser = ISO3166Parser()
iso3166_path = join(package_home(product_globals), 'iso3166')
iso3166_parser.parseAll(iso3166_path)

country_codes = [c[0] for c in iso3166_parser._countries]

##/code-section module-header

schema= Schema((
    LinesField('supported_countries',
            default= country_codes        ,
        widget=LargeMultiSelectionWidget
(
    label="Supported Countries",
    label_msgid='PloneMallCustomer_label_supported_countries',
    description='Enter a value for Supported Countries.',
    description_msgid='PloneMallCustomer_help_supported_countries',
    i18n_domain='PloneMallCustomer',
)        ,
        vocabulary='getAllCountries'    ),
    
    LinesField('preferred_countries',
            default= country_codes        ,
        widget=LargeMultiSelectionWidget
(
    label="Preferred Countries",
    label_msgid='PloneMallCustomer_label_preferred_countries',
    description='Enter a value for Preferred Countries.',
    description_msgid='PloneMallCustomer_help_preferred_countries',
    i18n_domain='PloneMallCustomer',
)        ,
        vocabulary='getAllCountries'    ),
    
    StringField('DefaultCountry',
            default="US"        ,
        widget=SelectionWidget
(
    label="Default Country",
    label_msgid='PloneMallCustomer_label_DefaultCountry',
    description='Enter a value for Default Country.',
    description_msgid='PloneMallCustomer_help_DefaultCountry',
    i18n_domain='PloneMallCustomer',
)        ,
        vocabulary='getAllCountries'    ),
    
    StringField('DefaultState',
            widget=StringWidget(
    label="Default State",
    label_msgid='PloneMallCustomer_label_DefaultState',
    description='Enter a value for Default State.',
    description_msgid='PloneMallCustomer_help_DefaultState',
    i18n_domain='PloneMallCustomer',
)    ),
    
    BooleanField('askLocationName',
            default="1"        ,
        widget=BooleanWidget(
    label="Ask Location?",
    label_msgid='PloneMallCustomer_label_askLocationName',
    description='Enter a value for Ask Location?.',
    description_msgid='PloneMallCustomer_help_askLocationName',
    i18n_domain='PloneMallCustomer',
)    ),
    
),
)


##code-section after-schema #fill in your manual code here
##/code-section after-schema

class AVS(UniqueObject,BaseContent):
    """
    Address Verification System
    """
    security = ClassSecurityInfo()
    __implements__ = (getattr(UniqueObject,'__implements__',()),) + (getattr(BaseContent,'__implements__',()),) + (IAVS,)


    # This name appears in the 'add' box
    archetype_name             = 'AVS'

    meta_type    = 'AVS' 
    portal_type  = 'AVS' 
    allowed_content_types      = [] 
    filter_content_types       = 0
    global_allow               = 0
    allow_discussion           = 0
    #content_icon               = 'AVS.gif'
    immediate_view             = 'base_view'
    default_view               = 'base_view'
    typeDescription            = "AVS"
    typeDescMsgId              = 'description_edit_avs'
    #toolicon                   = 'AVS.gif'

    schema = BaseSchema + \
             schema

    ##code-section class-header #fill in your manual code here
    ##/code-section class-header


    # tool-constructors have no id argument, the id is fixed
    def __init__(self, id=None):
        BaseContent.__init__(self,'portal_avs')
        
        ##code-section constructor-footer #fill in your manual code here
        ##/code-section constructor-footer



    #Methods
    #methods from Interface IAVS

    security.declareProtected("View", 'getCountries')
    def getCountries(self):
        '''
        Return a list of tuples, where the first tuple position is
        the coutry code (following the ISO3166 specification) and the
        second position is the Country name.
        '''

        ac = dict(self.getAllCountries())        
        pref = self.getPreferredCountries(ac)
        d_pref = dict(pref)
        supp = self.getSupported_countries()
        return pref + [(s, ac[s]) for s in supp if not d_pref.has_key(s)]



    security.declareProtected("View", 'getStatesForCountry')
    def getStatesForCountry(self,country):
        '''
        Return the States of a Country as a list of tuples, where the
        first tuple position if the State code (following the ISO3166
        specification) and the second position is the State name.
        '''
        
        l = iso3166_parser._states[country]
        return l or [('##', 'This Country does not have States.')]



    security.declareProtected("View", 'getSupportedCountries')
    def getSupportedCountries(self, ac = None):
        '''
        Lists the countries that are supported as a list of tuples
        (iso,countryname)
        '''
        
        if ac is None:
            ac = dict(self.getAllCountries())        
        ret = []
        for pc in self.getSupported_countries():
            ret.append((pc, ac[pc]))
        return ret



    security.declareProtected("View", 'getPreferredCountries')
    def getPreferredCountries(self, ac = None):
        '''
        Lists the countries that are preferred and appear first in
        the country list as a list of tuples (iso,countryname)
        '''
        
        if ac is None:
            ac = dict(self.getAllCountries())        
        ret = []
        for pc in self.getPreferred_countries():
            ret.append((pc, ac[pc]))
        return ret


    #manually created methods

    def getAllCountries(self):
        '''
        Return a list of tuples, where the first tuple position is the
        coutry code (following the ISO3166 specification) and the second
        position is the Coutry name.
        '''
        
        return iso3166_parser._countries



registerType(AVS,PROJECTNAME)
# end of class AVS

##code-section module-footer #fill in your manual code here
if register_avs:
   FactoryRegistry.registerMallFactory('AVS', AVS, (IAVS,))
##/code-section module-footer



