from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import *
from Products.PloneMallItem.BaseItem import BaseItem
from Products.PloneMallRegistry.IInventorial import IInventorial
from Products.PloneMallRegistry.util import SQLStorage
from Products.PloneMallRegistry.MallRegistry import FactoryRegistry
from Products.PloneMallRegistry.IMallItem import IMallItem
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import CMFCorePermissions
from config import PROJECTNAME, ITEM_STATUS

class InventoryItem(BaseContent,BaseItem):
    '''
    An Inventorial Item based on BaseItem.  This is meant to be an
    example of how to make an item inventorial.
    '''
    security = ClassSecurityInfo()
    portal_type = meta_type = 'InventoryItem' 
    archetype_name = 'InventoryItem'   #this name appears in the 'add' box 
    global_allow = 1

    __implements__ = getattr(BaseContent,'__implements__',()) + getattr(BaseItem,'__implements__',()) + (IInventorial,)

    ##code-section class-header #fill in your manual code here
    ##/code-section class-header

    schema=BaseSchema  + getattr(BaseItem,'schema',Schema(())) + Schema((
        TextField('description',
            searchable=1,
            widget=TextAreaWidget(description='Enter a value for short description.',
                description_msgid='PloneMallItem_help_short_description',
                i18n_domain='PloneMallItem',
                label='''Short Description''',
                label_msgid='PloneMallItem_label_short_description',
            ),
        ),
        
        TextField('fullDescription',
            searchable=1,
            widget=TextAreaWidget(description='Enter a value for full description.',
                description_msgid='PloneMallItem_help_full_description',
                i18n_domain='PloneMallItem',
                label='''Full Description''',
                label_msgid='PloneMallItem_label_full_description',
            ),
        ),
        
        ImageField('photo',
            sizes={'small':(80,80),'medium':(200,200),'large':(600,600)},
            storage=AttributeStorage(),
            widget=ImageWidget(description='Enter a value for photo.',
                description_msgid='PloneMallItem_help_photo',
                i18n_domain='PloneMallItem',
                label='Photo',
                label_msgid='PloneMallItem_label_photo',
            ),
        ),
        
        StringField(
            'status',
            vocabulary = ITEM_STATUS,
            widget=SelectionWidget(
                label = 'Status',
                description = 'Set the status of this item',
            ),
        ),
        IntegerField(
            'quantity',
            widget = IntegerWidget(
                label = 'Quantity',
                description = 'Enter the number of items available',
            ),
        ),
        
    ),
    )
        
    actions=  (
       {'action':      '''string:$object_url/base_view''',
        'id':          'view',
        'permissions': ('''View''',)
        },
        { 'id': 'inventory',
          'name': 'Inventory',
          'action': 'string:${object_url}/inventory_view',
          'permissions': (CMFCorePermissions.ModifyPortalContent,),
          'category':'object',
        },
    )
    
    def getShort_description(self):
        ''' for backward compatibility'''
        return self.Description()

    def isPurchaseableProduct(self, skipInterfaceCheck=False):
        '''
        Return true if the item is a purchaseable product.
        '''

        reg = getToolByName(self, 'portal_mall_registry')
        
        #for increasing performance:
        if skipInterfaceCheck:
            return 1
        
        return reg.objectSupportsInterface(self, IMallItem) #and 'SimpleItem' in reg.getIMallItem()
        

registerType(InventoryItem, PROJECTNAME)
FactoryRegistry.registerMallFactory('InventoryItem', InventoryItem, (IMallItem, IInventorial))


