from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import *
from Products.CMFCore.utils import UniqueObject
from Products.PloneMallRegistry.MallRegistry import FactoryRegistry
from Products.PloneMallRegistry.IInventoryManager import IInventoryManager
from Products.PloneMallRegistry.IInventorial import IInventorial
from zLOG import LOG, INFO, DEBUG
from Products.PloneMallInventory import PROJECTNAME



class InventoryManager(UniqueObject,BaseContent):
    '''
    Inventory Manager Tool
    '''
    security = ClassSecurityInfo()
    archetype_name = portal_type = meta_type = 'InventoryManager'
    global_allow = 0

    __implements__ = getattr(UniqueObject,'__implements__',()) + getattr(BaseContent,'__implements__',()) + (IInventoryManager,)

    schema= BaseSchema  + Schema((
    ),
    )

    # tool-constructors have no id argument, the id is fixed
    def __init__(self, id=None):
        BaseContent.__init__(self,'portal_inventory_manager')
        
    def isInventorialItem(self, item):
        """
        Method to determine if an object is inventorial
        """
        return IInventorial.isImplementedBy(item)
        
    def processOrder(self, order):
        """
        Process order and change quantity of inventorial items
        also set status if quantity goes to 0
        """
        # get the cart items
        cart_items = order.getCart().getContents()
        for cart_item in cart_items:
            item = cart_item.getMallItem()
            # check to see if items in order are inventorial
            if IInventorial.isImplementedBy(item):
                quantity = int(cart_item.getQuantity())
                # check that quantity is not more than available quantity
                # TODO: probably need to do this in the cart itself
                #if cart_quantity > item_quantity:
                #    quantity = item_quantity
                #else:
                #    quantity = cart_quantity
                self.removeFromStock(item, quantity)
    
    def addToStock(self, item, quantity):
        """
        Add quantity to the inventory
        """
        method = FactoryRegistry.getMallFactoryIdsOf('IInventoryMethod')
        
        ## TODO: support for multiple methods
        inv_method = method[0]
        imf = FactoryRegistry.getMallFactory(inv_method)
        im = imf()
        im.addToStock(item, quantity)
        
    def removeFromStock(self, item, quantity):
        """
        Remove quantity from the inventory
        """
        method = FactoryRegistry.getMallFactoryIdsOf('IInventoryMethod')
        
        ## TODO: support for multiple methods
        inv_method = method[0]
        imf = FactoryRegistry.getMallFactory(inv_method)
        im = imf()
        im.removeFromStock(item, quantity)
        
    def getAvailability(self, item):
        """
        Get the status of the item
        """
        method = FactoryRegistry.getMallFactoryIdsOf('IInventoryMethod')
        
        ## TODO: support for multiple methods
        inv_method = method[0]
        imf = FactoryRegistry.getMallFactory(inv_method)
        im = imf()
        im.getAvailability(item)
        
    def setAvailability(self, item, status):
        """
        Get the status of the item
        """
        method = FactoryRegistry.getMallFactoryIdsOf('IInventoryMethod')
        
        ## TODO: support for multiple methods
        inv_method = method[0]
        imf = FactoryRegistry.getMallFactory(inv_method)
        im = imf()
        im.setAvailability(item, status)

registerType(InventoryManager, PROJECTNAME)
FactoryRegistry.registerMallFactory('InventoryManager', InventoryManager, (IInventoryManager,))
