from Products.PloneMallRegistry.MallRegistry import FactoryRegistry
from Products.PloneMallRegistry.IInventoryMethod import IInventoryMethod
from zLOG import LOG, INFO, DEBUG
from config import PROJECTNAME


class SimpleInventoryMethod:
    """
    Simple Inventory method to inventory product by product
    """
    __implements__ = (IInventoryMethod,)
    
    def addToStock(self, item, quantity):
        """Add quantity to the inventory
        """
        current_quantity = item.getQuantity()
        new_quantity = current_quantity + quantity
        if current_quantity == 0 and new_quantity > 0:
            self.setAvailability(item, 'available')
        item.setQuantity(new_quantity)
        
    def removeFromStock(self, item, quantity):
        """Remove quantity from the inventory
        """
        current_quantity = item.getQuantity()
        new_quantity = current_quantity - quantity
        ## should we do this or return an error somehow?
        ## say there are 450 items and the person types 500 instead of 50
        if new_quantity < 0:
            new_quantity = 0
        item.setQuantity(new_quantity)
        # if quantity goes to zero set item to 'Sold Out'
        if item.getQuantity() == 0:
            self.setAvailability(item, 'sold-out')
        
    def getAvailability(self, item):
        """Get the status of the item
        """
        return item.getStatus()
        
    def setAvailability(self, item, status):
        """Set the status of the item
        """
        item.setStatus(status)

FactoryRegistry.registerMallFactory('SimpleInventoryMethod', SimpleInventoryMethod, (IInventoryMethod,))
