# File: OrderManager.py
"""\
Order Manager

"""
# Copyright (c) 2005 by Weimar Desenvolvimento e Consultoria em Informatica
# Ltda (Brasil), jens quadrat - AGENTUR FUER NEUE MEDIEN OEG (Austria),
# BlueDynamics KEG (Austria), Zettai.net LCC (United States)
# Generator: ArchGenXML Version 1.4 devel 2 http://sf.net/projects/archetypes/
#
# GNU General Public Licence (GPL)
# 
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA  02111-1307  USA
#
__author__  = '''Fabiano Weimar dos Santos <xiru@xiru.org>, Jens Klein
<jensens@jensquadrat.de>, Phil Auersperg <phil@bluedynamics.org>'''
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *


from Products.PloneMallRegistry.IOrderManager import IOrderManager



from Products.PloneMallOrder.config import *


from Products.CMFCore.utils import UniqueObject

    
##code-section module-header #fill in your manual code here
from config import *
from Products.PloneMallRegistry.MallRegistry import FactoryRegistry
from Products.CMFCore.utils import getToolByName
from AccessControl.unauthorized import Unauthorized
from exceptions import *
##/code-section module-header

schema= Schema((
    IntegerField('next_order_id',
        default="1",
        widget=IntegerWidget(
            label="Next Order ID",
            label_msgid='PloneMallOrder_label_next_order_id',
            description='Enter a value for Next Order ID.',
            description_msgid='PloneMallOrder_help_next_order_id',
            i18n_domain='plonemallorder',
        ),
        required=1,
    ),
    
),
)


##code-section after-schema #fill in your manual code here
##/code-section after-schema

class OrderManager(UniqueObject,BaseBTreeFolder):
    """
    Order Manager
    """
    security = ClassSecurityInfo()
    __implements__ = (getattr(UniqueObject,'__implements__',()),) + (getattr(BaseBTreeFolder,'__implements__',()),) + (IOrderManager,)


    # This name appears in the 'add' box
    archetype_name             = 'OrderManager'

    portal_type = meta_type    = 'OrderManager' 
    allowed_content_types      = ['Order', 'MailOrder'] 
    filter_content_types       = 1
    global_allow               = 0
    allow_discussion           = 0
    #content_icon               = 'OrderManager.gif'
    immediate_view             = 'base_view'
    default_view               = 'base_view'
    typeDescription            = "OrderManager"
    typeDescMsgId              = 'description_edit_ordermanager'
    #toolicon                   = 'OrderManager.gif'

    schema = BaseFolderSchema + \
             schema

    ##code-section class-header #fill in your manual code here
    ##/code-section class-header


    # tool-constructors have no id argument, the id is fixed
    def __init__(self, id=None):
        BaseBTreeFolder.__init__(self,'portal_ordermanager')
        
        ##code-section constructor-footer #fill in your manual code here
        ##/code-section constructor-footer



    #Methods
    #methods from Interface IOrderManager

    security.declareProtected("Buy item", 'addOrder')
    def addOrder(self):
        '''
        Add a new Order and returns the order id.
        '''

        # TODO: This is a really big method. In the future, this
        # method must be splitted in several methods to make easier
        # class derivations.
        
        customer_information_id = default_customer_information_id
        mall_registry=getToolByName(self,'portal_mall_registry')

        self._CheckoutFrontController()

        pm = getToolByName(self, 'portal_membership')


        self.manage_beforeCreateOrder()

        # calculate the next order id
        order_id = self.getNextOrderId()
        order_number=order_id
        

        # create Order (o)
        of = FactoryRegistry.getMallFactory('Order')
        o = of(str(order_id))
        self._setObject(str(order_id), o)
        o = getattr(self, str(order_id))

        if hasattr(self,'calcOrderNumberScript'):
            order_number=self.calcOrderNumberScript(o)

        o.setTitle(order_number)

        self.manage_beforeProcessCustomerInformation()

        # create CustomerInformation (ci)
        mf = pm.getHomeFolder()
        if mf:
            ci = mf._getOb(customer_information_id, None)
        else:
            ci=None
            mf=o #store the address information with the order
            
        if ci is None:
            cif = mall_registry.getMallFactoriesOf('ICustomerInformation')[0]
            ci = cif(customer_information_id)
            if mf:
                mf._setObject(customer_information_id, ci)
                ci = getattr(mf, customer_information_id)
                
        
        
        self.manage_afterProcessCustomerInformation(ci, o)

        self.manage_beforeProcessAddressInformationSet()

        # create AddressInformationSet (ais)
        if mf:
            ais = mf._getOb(default_address_information_set_id, None)
        else:
            ais=None
            mf=o #store the address information with the order
            
        if ais is None:
            aisf = FactoryRegistry.getMallFactory('AddressInformationSet')
            ais = aisf(default_address_information_set_id)
            if mf:
                mf._setObject(default_address_information_set_id, ais)
                ais = getattr(mf, default_address_information_set_id)

        ci.addReference(ais, 'address')

        self.manage_afterProcessAddressInformationSet(ais)

        # For now, the implementation supports only one address
        # information. I a future version, the customer will support
        # more than one address information. It's planned on the
        # model, but a bit hard to implement in a resonable way.

        self.manage_beforeProcessAddressInformation()

        # create AddressInformation (ai)
        ai = ais._getOb(default_address_information_id, None)
        if ai is None:
            aif = FactoryRegistry.getMallFactory('AddressInformation')
            ai = aif(default_address_information_id)
            ais._setObject(default_address_information_id, ai)
            ai = getattr(ais, default_address_information_id)

        self.manage_afterProcessAddressInformation(ai)
        
        # create AddressInformation for shipping info (aiship)
        aiship = ais._getOb(default_shipping_address_information_id, None)
        if aiship is None:
            aif = FactoryRegistry.getMallFactory('AddressInformation')
            aiship = aif(default_shipping_address_information_id)
            ais._setObject(default_shipping_address_information_id, aiship)
            aiship = getattr(ais, default_shipping_address_information_id)

        self.manage_afterProcessAddressInformation(aiship)

        session = self.REQUEST.SESSION

        self.manage_beforeProcessSessionVariables(session)

        # process the customer information from SESSION
        customer = session.get('customer', {})
        #dirty update 
        #TODO: use update and have a worflow that allows an anon owner to edit
        # the customer data at the initial state
        for name in customer.keys():
            field = ci.Schema().get(name, None)
            if field is None:
                continue
            method = field.getMutator(ci)
            method(customer[name])
            
        ci.update(**customer)

        # process the customer address information from SESSION
        customer_address = session.get('customer_address', {})

        #dirty update 
        #TODO: use update and have a worflow that allows an anon owner to edit
        # the customer data at the initial state
        for name in customer_address.keys():
            field = ai.Schema().get(name, None)
            if field is None:
                continue
            method = field.getMutator(ai)
            method(customer_address[name])

        ai.update(**customer_address)
        
        # process the customer shipping address address information from SESSION
        customer_shipping_address = session.get('add_customer_shipping_address', {})
        aiship.update(**customer_shipping_address)

        if not hasattr(o,ci.getId()):
            #copy the customer info into the order
            cic_=ci._getCopy(o)
            o._setObject(cic_.getId(),cic_)
            cic=getattr(o,cic_.getId())
        else:
            cic = ci
        
        if not hasattr(o,ais.getId()):
            #copy the address into the order
            aisc_=ais._getCopy(o)
            o._setObject(aisc_.getId(),aisc_)
            aisc=getattr(o,aisc_.getId())
            cic.setAddress(aisc)

        else:
            aisc = ais
        
        aic=getattr(aisc,default_address_information_id)
        # hook customer information
        o.setCustomer(cic)

        # hook customer address information
        o.setCustomerAddress(aic)

        # hook cart
        pcm = getToolByName(self, 'portal_cartmanager')
        o.setCart(pcm.getOrCreateCart())

        # process shipping
        o.setHas_shipping(has_shipping)
        o.setHas_shipping_address_by_item(has_shipping and \
                                          has_shipping_address_by_item)

        if has_shipping:

            self.manage_beforeProcessShipping(o)

            # process the shipping method from SESSION
            shipping_method = session.get('shipping_method', None)

            # Shipping Method (sm)
            smf = FactoryRegistry.getMallFactory(shipping_method)
            sm = smf()
            o = sm.processShipping(o)

            self.manage_afterProcessShipping(o)

        # process tax
        o.setHas_tax(has_tax)
        o.setHas_tax_by_item(has_tax and has_tax_by_item)

        # get crazy with the cheeze whiz
        # hacking in the ICE discount rate
        discount = float(session.get('discount_total', 0.0))
        if discount > 0.0:
            o.setDiscount(str(discount))

        if has_tax:

            self.manage_beforeProcessTax(o)

            # process the tax method from SESSION
            tax_method = session.get('tax_method', None)

            # Tax Method (tm)
            tmf = FactoryRegistry.getMallFactory(tax_method)
            tm = tmf()
            o = tm.processTax(o)

            self.manage_afterProcessTax(o)

        self.manage_afterCreateOrder(o)

        # process payment
        o.setHas_payment(has_payment)

        if has_payment:

            self.manage_beforeProcessPayment(o)

            # process the payment method from SESSION
            payment_method = session.get('payment_method', None)

            # Payment Method (pm)
            pmf = FactoryRegistry.getMallFactory(payment_method)
            pm = pmf()
            pm.processPayment(o)

            self.manage_afterProcessPayment(o)


        return order_id



    security.declareProtected("View", 'getOrderById')
    def getOrderById(self, order_id):
        '''
        Return an Order but its order id.
        '''

        return self._getOb(str(order_id), None)
    
    def getValue(self,name,default=None):
        #helper function to set helper values in the OrderManager
        return getattr(self,name,default)

    def setValue(self,name,value):
        setattr(self,name,value)
        

    #manually created methods

    def manage_beforeProcessTax(self, order):
        pass


    def manage_afterProcessAddressInformation(self, ai):
        pass


    def manage_beforeProcessSessionVariables(self, session):
        pass


    def manage_beforeProcessPayment(self, order):
        pass


    def manage_beforeProcessCustomerInformation(self):
        pass


    def manage_beforeCreateOrder(self):
        pass


    def manage_afterFrontController(self):
        pass


    def manage_afterProcessAddressInformationSet(self, ais):
        pass


    def manage_beforeProcessAddressInformationSet(self):
        pass


    def manage_beforeFrontController(self):
        pass


    def manage_afterProcessShipping(self, order):
        pass


    def _CheckoutFrontController(self):
        '''
        It is a "Front Controller" Design Pattern, an elegant solution
        to control the navigation on checkout. The main idea here is
        point all Controller Page Template (those that finish with
        _redirect) to here and decide what is missing to create an
        Order based on REQUEST and SESSION variables. If something is
        missing, an excepting raises and the Python Script checkout.py
        decides where to redirect.
        '''

        self.manage_beforeFrontController()

        pcm = getToolByName(self, 'portal_cartmanager')
        cart = pcm.getOrCreateCart()
        if not len(cart.getContents()):
            raise EmptyCart

        # check Anonymous user
        pm = getToolByName(self, 'portal_membership')
        #if pm.isAnonymousUser():
        #    raise Unauthorized

        session = self.REQUEST.SESSION

        # import pdb; pdb.set_trace()

        # check the customer information from SESSION
        customer = session.get('customer', {})
        try:
            assert(customer.get('firstname', None) is not None)
            assert(customer.get('lastname', None) is not None)
            assert(customer.get('email', None) is not None)
        except AssertionError:
            raise MissingCustomer

        # check the customer address information from SESSION
        customer_address = session.get('customer_address', {})
        try:
#            assert(customer_address.get('location_name', None) is not None)
            assert(customer_address.get('address1', None) is not None)
#            assert(customer_address.get('address2', None) is not None)
            assert(customer_address.get('postal_code', None) is not None)
            assert(customer_address.get('city', None) is not None)
#            assert(customer_address.get('state', None) is not None)
            assert(customer_address.get('country', None) is not None)
        except AssertionError:
            raise MissingCustomerAddress

        # check the shipping method from SESSION
        if has_shipping:
            shipping_method = session.get('shipping_method', None)
            if shipping_method is None:
                raise MissingShippingMethod

        # check the payment method from SESSION
        if has_payment:
            payment_method = session.get('payment_method', None)
            if payment_method is None:
                raise MissingPaymentMethod

        # check the tax method from SESSION
        if has_tax:
            tax_method = session.get('tax_method', None)
            if tax_method is None:
                raise MissingTaxMethod

        # check the customer confirmation
        if self.REQUEST.get('order_confirmation', None) is None:
            raise MissingOrderConfirmation

        self.manage_afterFrontController()


    def getNextOrderId(self):
        #TODO: getNextOrderId should use UIDs or other non-guessable numbers
        currid=self.getNext_order_id()
        nextid=int(currid) + 1
        self.setNext_order_id(nextid)
        self.last_order_id=currid
        return nextid
    
    def manage_afterProcessTax(self, order):
        pass


    def manage_afterProcessPayment(self, order):
        pass


    def manage_beforeProcessAddressInformation(self):
        pass


    def listAvailableCountries(self):
        pass


    def manage_beforeProcessShipping(self, order):
        pass


    def manage_afterProcessCustomerInformation(self, ci, order=None):
        if hasattr(self,'afterProcessCustomerInformation'):
            self.afterProcessCustomerInformation(ci, order)


    def manage_afterCreateOrder(self, order):
        pass



registerType(OrderManager,PROJECTNAME)
# end of class OrderManager

##code-section module-footer #fill in your manual code here
if register_order:
    FactoryRegistry.registerMallFactory('OrderManager', OrderManager, (IOrderManager,))
##/code-section module-footer



