from fixedpoint import FixedPoint

try:
   from Products.Archetypes.public import DisplayList

# TODO: We need a exchange service to convert one currency to another.
# TODO: We need a tool to represent currency codes (if possible, by country). See ISO-4217 specification.

   VOC_CURRENCY = DisplayList((
      ('USD', 'US$'), # U.S. Dollars
      ('EUR', 'Eur'), # Euros
#  ('GBP', 'GBP'), # Pounds Sterling
#  ('CAD', 'CAD'), # Canadian Dollars
#  ('JPY', 'JPY'), # Yen
    ))
except ImportError:
    pass

class Currency:
    '''
    FixedPoint wrapper to represent Currency
    '''

    def __init__(self, currency = None, value = '0.00'):
        self._currency = currency
        self._value = FixedPoint(value)

    def compareCurrency(self,c1,c2):
        if c1: c1=c1.lower()
        if c2: c2=c2.lower()
        return c1==c2

    def __add__(self, other):
        
        # if one of the summands has no currency set, lets set it
        # automatically by the other's currency
        currency=self._currency or other._currency

        if type(self) != type(other):
            raise TypeError("You should sum a Currency object with another Currency object.")

        if self._currency and other._currency and not self.compareCurrency(self._currency , other._currency):
            raise TypeError("You should not sum different kinds of currency.")

        return Currency(currency, str(self._value + other._value))

    def __mul__(self, other):
        return Currency(self._currency, str(self._value * FixedPoint(str(other))))

    def getCurrencySymbol(self):
        '''
        Return the currency symbol.
        '''
        return self._currency

    def getValue(self):
        '''
        Return the currency value.
        '''
        return str(self._value)

    def getFormatedValue(self):
        '''
        Return a formated string with the currency symbol and the
        currency value.
        '''
        return "%s %s" % (self._currency, self._value)

    def __repr__(self):
        return self.getFormatedValue()

    def __float__(self):
        return float(self.getValue())

if __name__ == '__main__':
    c1 = Currency('USD', '10.99')
    print c1
    c2 = Currency('USD', '12.00')
    print c2
    print c1 + c2
    print c1 * 3
    print c1 * 3.0
    print c1 * '3'
    print c1 * '3.0'

    c2 = Currency('USD','100')
    print 'float(c2):',float(c2)
