# File: MallRegistryTool.py
"""\
A tool to store PloneMall interface registry settings, variable schemas and
general settings.

"""
# Copyright (c) 2005 by Weimar Desenvolvimento e Consultoria em Informatica
# Ltda (Brasil), jens quadrat - AGENTUR FUER NEUE MEDIEN OEG (Austria),
# BlueDynamics KEG (Austria), Zettai.net LCC (United States)
# Generator: ArchGenXML Version 1.4 devel 4 http://sf.net/projects/archetypes/
#
# GNU General Public Licence (GPL)
# 
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA  02111-1307  USA
#
__author__  = '''Fabiano Weimar dos Santos <xiru@xiru.org>, Jens Klein
<jensens@jensquadrat.de>, Phil Auersperg <phil@bluedynamics.org>'''
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *



from Products.Archetypes.VariableSchemaSupport import VariableSchemaSupport

from Products.PloneMallRegistry.config import *

from Products.CMFCore.utils import UniqueObject

    
##code-section module-header #fill in your manual code here
from Products.PloneMallRegistry.MallRegistry import FactoryRegistry
from Products.PloneMallRegistry.Currency import VOC_CURRENCY
##/code-section module-header

schema= Schema((
    StringField('DefaultCurrency',
        default="USD",
        widget=SelectionWidget
        (
            label="Default Currency",
            label_msgid='PloneMallRegistry_label_DefaultCurrency',
            description='Enter a value for Default Currency.',
            description_msgid='PloneMallRegistry_help_DefaultCurrency',
            i18n_domain='plonemallregistry',
        ),
        required=1,
        vocabulary=VOC_CURRENCY,
    ),
    
    StringField('Country',
        widget=StringWidget(
            label='Country',
            label_msgid='PloneMallRegistry_label_Country',
            description='Enter a value for Country.',
            description_msgid='PloneMallRegistry_help_Country',
            i18n_domain='plonemallregistry',
        ),
    ),
    
),
)


##code-section after-schema #fill in your manual code here
##/code-section after-schema

class MallRegistryTool(UniqueObject,VariableSchemaSupport,BaseContent):
    """
    A tool to store PloneMall interface registry settings, variable
    schemas and general settings.
    """
    security = ClassSecurityInfo()
    __implements__ = (getattr(UniqueObject,'__implements__',()),) + (getattr(VariableSchemaSupport,'__implements__',()),) + (getattr(BaseContent,'__implements__',()),)


    # This name appears in the 'add' box
    archetype_name             = 'MallRegistryTool'

    portal_type = meta_type    = 'MallRegistryTool' 
    allowed_content_types      = [] 
    filter_content_types       = 0
    global_allow               = 0
    allow_discussion           = 0
    #content_icon               = 'MallRegistryTool.gif'
    immediate_view             = 'base_view'
    default_view               = 'base_view'
    typeDescription            = "MallRegistryTool"
    typeDescMsgId              = 'description_edit_mallregistrytool'
    #toolicon                   = 'MallRegistryTool.gif'

    schema = BaseSchema + \
             schema

    ##code-section class-header #fill in your manual code here
    ##/code-section class-header


    # tool-constructors have no id argument, the id is fixed
    def __init__(self, id=None):
        BaseContent.__init__(self,'portal_mall_registry')
        
        ##code-section constructor-footer #fill in your manual code here
        ##/code-section constructor-footer



    #Methods
    #manually created methods

    def getSupportedFactoryNames(self,interface):
        ''' returns the names of factories enablied for an interface '''
        return self.Schema().getField(FactoryRegistry.getInterfaceDefinition(interface).getName()).getAccessor(self)()        


    def classSupportsInterface(self, klass, interface):
        '''
        Determines if a class supports a certain interface in the registry.
        '''

        supportedFactoryNames=self.getSupportedFactoryNames(interface)
        
        return klass.__name__ in supportedFactoryNames and FactoryRegistry.classSupportsInterface(klass,interface)


    def getMallFactoriesOf(self, interface_definition):
        '''
        Return a list of factories that implements a PloneMall interface.

        @param interface_definition PloneMall interface definition.
        '''

        supportedFactoryNames=self.getSupportedFactoryNames(interface_definition)
        return [f for f in FactoryRegistry.getMallFactoriesOf(interface_definition) if f.getName() in f.getName() in supportedFactoryNames]


    def objectSupportsInterface(self, obj, interface):
        '''
        Determines if an object supports a certain interface in the registry.
        '''

        return self.classSupportsInterface(obj.__class__, interface)


    def getMallFactoryIdsOf(self, interface_definition):
        '''
        Return a list of factory names that implements a PloneMall interface.

        @param interface_definition PloneMall interface definition.
        '''

        return [r.id for r in self.getMallFactoriesOf(interface_definition)]


    def getSchema(self):
        newschema = self.schema
        for i in FactoryRegistry.getInterfaces():
            imps = FactoryRegistry.getMallFactoryIdsOf(i)
            newschema = newschema + Schema((
                    LinesField(
                        i.__name__,
                        schemata = 'interfaces',
                        default = imps,
                        vocabulary = imps,
                        multiValued = 1,
                        widget = MultiSelectionWidget(),
                        ),
                    ))
        return self.schema+newschema



registerType(MallRegistryTool,PROJECTNAME)
# end of class MallRegistryTool

##code-section module-footer #fill in your manual code here
##/code-section module-footer



