import urllib
import httplib as httpslib
import string
from datetime import date
import time
from Products.PloneMallShipping.config import PROJECTNAME
from zLOG import LOG, DEBUG, INFO


# meter: 1055620

class FedExRate:
    """
    A class to calculate the cost of shipping on a per item basis
    """
    
    def __init__(self, referer):
        self.host = "gatewaybeta.fedex.com"
        self.referer = referer
        self.accountNumber = "255871609"
        self.meterNumber   = "1055620"
        
        # the error codes #
        self.error = {
            '0038' : 'Invalid tracking number.',
            '0105' : 'Internal FedEx Server error (0105).',
            '801D' : 'Account number required.',
            '0911' : 'Resources are unavailable to service your request.',
            '1010' : 'A FedEx tracking number or Reference Data field must be included in the tracking request.',
            '1020' : 'Missing Transaction Type. Transaction Type (tag #0) is a required field.',
            '1040' : 'Missing Account Number. Account Number (tag #10) is required for Alternate-ReferenceTracking.',
            '1050' : 'Missing Carrier Code. Carrier Code (tag #3025) is required for Alternate-Reference Tracking.',
            '1060' : 'Invalid Carrier Code. If Carrier Code (tag #3025) is specified, it must be FDXG or FDXE.',
            '1070' : 'A FedEx Reference type (tag #1453) is required for Alternate-Reference Tracking.',
            '1080' : "Track Reference Type must indicate '1' for Express Track-by-Reference.",
            '1101' : 'Airbill already deleted',
            '1102' : 'Internal FedEx domestic ship error (1102)',
            '1103' : 'Internal FedEx domestic ship error (1103)',
            '1104' : 'Internal FedEx domestic ship error (1104)',
            '1105' : 'Internal FedEx domestic ship error (1105)',
            '1106' : 'Internal FedEx domestic ship error (1106)',
            '1107' : 'Internal FedEx domestic ship error (1107)',
            '1108' : 'No courtesy rates provided for bill recipient, 3 rd party, or credit card payment types.',
            '1109' : 'Internal FedEx domestic ship error (1109)',
            '110A' : 'Internal FedEx domestic ship error (110A)',
            '110B' : 'Internal FedEx domestic ship error (110B)',
            '1200' : 'Service type does not allow alcohol',
            '1201' : 'Payment Type not allowed with COD.',
            '1202' : 'No COD allowed for bill third party',
            '1203' : 'Unknown or invalid transaction',
            '1204' : 'Internal FedEx domestic ship error (1204).',
            '1205' : 'Service type does not allow inaccessible dangerous goods',
            '1206' : 'Service type cannot have declared value greater than $500',
            '1207' : 'Shipment exceeds maximum Declared Value',
            '1208' : 'Service type cannot have declared value lower than $250',
            '1209' : 'Selected Service requires dim length, width and height.',
            '120A' : 'Girth + length exceeds maximum of 84" for service type',
            '120B' : 'Length exceeds maximum for service type',
            '120C' : 'Service type does not allow accessible dangerous goods',
            '120D' : 'Express Saver service not allowed for Bill Recipient/Bill Third Party',
            '120E' : 'Minimum dimensions not met for service type: length 7 inches, width 4 inches, height 2 inches',
            '120F' : 'Recipient address and postal code required',
            '1210' : 'Service type does not allow dry ice',
            '1211' : 'Puerto Rico destinations not valid via Domestic Ship -- use International Ship',
            '1212' : 'Hold At Location requires complete hold address including postal code',
            '1213' : 'Release specified but no authorization number provided',
            '1214' : 'COD service not available',
            '1215' : 'Economy Letter service not available',
            '1216' : 'Release not allowed on dangerous goods shipment',
            '1217' : 'No Hold At Location with Inside/Residential Delivery',
            '1218' : 'Service requires US domestic origin.',
            '1219' : 'Invalid package count',
            '121A' : 'Saturday delivery to Hawaii destinations not available',
            '121B' : 'No hold-at-location for service type on Saturday',
            '121C' : 'No Saturday delivery for freight',
            '121D' : 'Saturday delivery allowed only for Thursday ship date on economy service',
            '121E' : 'Saturday delivery allowed only for Friday ship date on priority service',
            '121F' : 'Service type does not allow Saturday delivery',
            '1220' : 'No Saturday delivery available',
            '1221' : 'Saturday delivery to Alaska not valid',
            '1222' : 'Saturday pick-up only available on Saturday ship date',
            '1223' : 'Saturday pick-up for Hawaii origins not available',
            '1224' : 'No Saturday pick-up for freight shipments',
            '1225' : 'No Saturday pickup for service type',
            '1226' : 'Service type not supported or defined',
            '1227' : 'Hawaii origin/destination not allowed for service type',
            '1228' : 'Service does not allow dimmed packaging',
            '1229' : 'Service type not valid for this destination',
            '122A' : 'Internal FedEx domestic ship error (122A).',
            '122B' : 'Internal FedEx domestic ship error (122B).',
            '122C' : 'Submitted weight evaluates to 0',
            '122D' : 'Dry ice specified but dry ice weight evaluates to 0',
            '122E' : 'Saturday delivery and Sunday delivery are mutually exclusive',
            '122F' : 'Package weight is less than dry ice weight',
            '1230' : 'Invalid weight for service type -- exceeds upper limit',
            '1231' : 'Invalid weight for service type -- below lower limit',
            '1232' : 'Weight exceeds maximum for service type',
            '1233' : 'Length or (girth + length) does not meet minimums for submitted weight and service type.',
            '1234' : 'COD not available for service type',
            '1235' : 'Invalid postal code format',
            '1236' : 'Internal FedEx domestic ship error (1236).',
            '1237' : 'Service not available to this postal code',
            '1238' : 'Drop-off and Saturday pick-up mutually exclusive',
            '1239' : 'Shipper account number must be provided for Bill Sender',
            '123A' : 'Saturday pick-up for Puerto Rico origins not available',
            '123B' : 'Invalid current date',
            '123C' : 'Invalid future date',
            '123D' : 'No Inside Pickup with service type',
            '123E' : 'No Inside Delivery with service type',
            '123F' : 'Sunday shipment should be marked future day',
            '1240' : 'Length, width, height exceeds the maximum 70 inches, 48 inches, 48 inches for service',
            '1241' : 'Invalid date/time format',
            '1242' : 'No COD allowed',
            '1243' : 'Shipment originated from CAN version',
            '1244' : 'COD Amount must be between 0 and $9,999,999.99',
            '1245' : 'Invalid payment type',
            '1246' : 'Release not allowed on COD shipment',
            '1247' : 'Required address field missing: Contact name',
            '1248' : 'Required address field missing: Company',
            '1249' : 'Required address field missing: Address1',
            '124A' : 'Required address field missing: Address2',
            '124B' : 'Required address field missing: City',
            '124C' : 'Required address field missing: State',
            '124D' : 'Required address field missing: ZIP code',
            '124E' : 'Required address field missing: Phone number',
            '124F' : 'Required address field missing: Country',
            '1250' : 'Bill recipient/Bill third party not allowed for service',
            '1251' : 'COD flags/amount cannot be set for non-COD shipment',
            '1252' : 'Account number field must be numeric',
            '1253' : 'Height supplied exceeds height maximum for service',
            '1254' : 'Width supplied exceeds width maximum for service',
            '1255' : 'Alaska origin/destination not allowed on service type',
            '1256' : 'Puerto Rico origin not allowed on service type',
            '1257' : 'Invalid credit card number',
            '1258' : 'Invalid credit card type',
            '1259' : 'Invalid credit card expiration date',
            '125A' : 'Invalid transaction request type',
            '125B' : 'Invalid rate quote payment type',
            '125C' : 'Credit card authorization failed',
            '125D' : 'Unable to authorize credit card number',
            '125E' : 'A/R balance exceeds credit limit',
            '1260' : 'Puerto Rico origin not allowed for payment type',
            '1261' : 'The selected service type is not allowed with Emerge.',
            '1262' : 'Sunday delivery for Hawaii origins not available',
            '1263' : 'Sunday delivery for Puerto Rico origins not available',
            '1264' : 'No COD allowed with Sunday delivery',
            '1265' : 'No hold at location allowed with Sunday delivery',
            '1266' : 'Saturday pickup with given destination not allowed with Sunday delivery',
            '1267' : 'Invalid customer number',
            '1268' : 'Invalid drop off flag option',
            '126A' : 'Emerge Shipment ID is invalid.',
            '126B' : 'Emerge Merchant ID is invalid.',
            '126D' : 'COD shipments are not allowed with Emerge.',
            '126E' : 'Dry Ice shipments not allowed with Emerge.',
            '126F' : 'Signature release is not allowed with Emerge.',
            '1270' : 'Emerge not allowed with freight service in H3 delivery areas.',
            '1271' : 'Sunday delivery is not allowed with Emerge.',
            '1272' : 'COD Add Freight Flag must = 1 when Rate Indicator = 1',
            '1277' : 'The specified service is not available with returns.',
            '1278' : 'Declared value cannot exceed $1000 for Express returns.',
            '1279' : 'Invalid pre-assigned tracking number.',
            '127A' : 'Invalid pre-assigned form ID.',
            '127B' : 'Alternate Sender not allowed.',
            '1300' : 'Internal FedEx domestic ship error (1300).',
            '1301' : 'Internal FedEx domestic ship error (1301).',
            '1302' : 'Internal FedEx domestic ship error (1302).',
            '1303' : 'Internal FedEx domestic ship error (1303).',
            '1311' : 'For Puerto Rico to US shipments please use Domestic ship',
            '1422' : 'Invalid transaction number',
            '1423' : 'Hold At Location postal code must be 5 or 9 numeric digits',
            '1424' : 'Sender Address country and Alternate Sender Address country must be the same.',
            '1425' : 'Alternate Sender Address postal code must be 5 or 9 digits.',
            '1426' : 'Alternate Sender Address Phone Number must be 10 digits.',
            '1427' : 'Alternate Sender Address postal code must be six digits.',
            '1428' : 'Invalid Alternate Sender Address postal code format for this country.',
            '1429' : 'Invalid Alternate Sender Address phone number length - cannot exceed 15 digits.',
            '142A' : 'Invalid Alternate Sender Address Postal Code.',
            '142B' : "Invalid AES filing status value -- must be 'S' for self-filer.",
            '142C' : 'AES or FTSR Exemption required.',
            '1450' : 'Canada Hold At Location requires province',
            '1451' : 'Canada Hold AT Location postal code must be six characters <ananan>',
            '1452' : 'US Hold At Location requires state',
            '1523' : 'Canada postal code must be six characters <ananan>',
            '1524' : 'Canada hold at location postal code must be six characters <ananan>',
            '1525' : 'Canada importer postal code must be six characters <ananan>',
            '1526' : 'Canada recipient postal code must be six characters <ananan>',
            '1527' : 'Invalid Canada sender postal code format',
            '1528' : 'Invalid Canada recipient postal code format',
            '1529' : 'Invalid Canada hold at location postal code format',
            '152A' : 'Invalid Canada importer postal code format',
            '152B' : 'Sender email address required for Ship Alert Notification.',
            '1555' : 'Saturday delivery to [ ZIP ] not allowed',
            '1556' : 'Saturday pickup from [ ZIP ] not allowed',
            '1700' : 'Incomplete commodity information',
            '1701' : 'Invalid description of contents',
            '1702' : 'Invalid country of manufacture for line 1',
            '1703' : 'Invalid country of manufacture for line 2',
            '1704' : 'Invalid country of manufacture for line 3',
            '1705' : 'Invalid country of manufacture for line 4',
            '1706' : 'Customs value must be 0.00 or 1.00',
            '1800' : 'Electronic Commercial Invoice not allowed for document shipment',
            '1801' : 'Ultimate destination country required for commercial invoice',
            '1802' : 'Terms of sale required for commercial invoice',
            '1803' : 'Recipient country does not allow electronic commercial invoice',
            '1804' : 'Complete importer information is required for electronic commercial invoice',
            '1805' : 'Importer state/province is required for commercial invoice',
            '1806' : 'Importer postal code is required for commercial invoice',
            '1808' : 'Importer postal code must be 5 or 9 numeric digits',
            '1F20' : 'ZIP code must be 5 or 9 numeric digits',
            '1F21' : 'Sender postal code must be 5 or 9 numeric digits',
            '1F22' : 'Recipient postal code must be 5 or 9 numeric digits',
            '1F25' : 'Ship date must be 8 numeric digits',
            '1F30' : 'Recipient phone must be 10 digits',
            '1F31' : 'Sender phone must be 10 digits',
            '1F32' : 'Recipient phone for US/Canada must be 10 digits',
            '1F33' : 'Sender phone for US/Canada must be 10 digits',
            '1F34' : 'Phone number for US/Canada must be 10 digits',
            '1F35' : 'Sender phone number length cannot be more than 15 digits',
            '1F36' : 'Recipient phone number length cannot be more than 15 digits',
            '1F37' : 'Emerge shipment ID is required since Emerge merchant ID is present.',
            '1F38' : 'Emerge merchant ID is required since Emerge shipment ID is present.',
            '1F39' : 'Emerge shipment ID length exceeds the maximum length of 15 characters.',
            '1F40' : 'Emerge merchant ID length exceeds the maximum length of 15 characters.',
            '1F55' : "Account must match owner's state, postal code, and country",
            '2500' : 'Client software version not supported',
            '2501' : 'Internal FedEx account validation error (2501)',
            '2502' : 'Invalid transaction type',
            '2503' : 'Meter number not found',
            '2504' : 'Valid FedEx account number required for bill recipient or bill 3 rd party',
            '2520' : 'Account has been marked for deletion',
            '2521' : 'Account has been marked cash-only',
            '2522' : 'Account is international (not US or Canada)',
            '2523' : 'Account is non-revenue',
            '4035' : 'Unable to rate (4035)',
            '4036' : 'Non Revenue Account',
            '4040' : 'Airbill was not found.',
            '4049' : 'Incorrect software revision',
            '4050' : 'Incorrect software version',
            '4051' : 'Unable to rate (4051)',
            '4052' : 'Invalid rate request',
            '4053' : 'Invalid meter number',
            '4054' : 'Invalid customer account number',
            '4055' : 'Invalid shipping date',
            '4056' : 'Invalid service type',
            '4057' : 'Invalid recipient postal code',
            '4058' : 'Invalid state code',
            '4059' : 'Invalid package count',
            '4060' : 'Invalid weight for selected service',
            '4061' : 'Invalid declared value',
            '4062' : 'Invalid length',
            '4063' : 'Invalid width',
            '4064' : 'Invalid height',
            '4065' : 'Invalid shipment handling',
            '4066' : 'Invalid shipment handling',
            '4067' : 'Package dimensions are missing',
            '4068' : 'Shipment too small or too large',
            '4069' : 'City center account number',
            '406A' : 'Invalid sender postal code',
            '406B' : 'Inside pickup unavailable',
            '406C' : 'Inside delivery unavailable',
            '406D' : 'Invalid credit card information',
            '406E' : 'Invalid credit card expiration date',
            '4070' : 'Account number marked for delete',
            '4071' : 'Cash only account',
            '4072' : 'Postal code is neither in US nor in Canada',
            '4073' : 'Unable to rate (4073)',
            '4074' : 'Unable to rate (4074)',
            '4075' : 'Unable to rate (4075)',
            '4076' : 'Unable to rate (4076)',
            '4077' : 'Unable to rate (4077)',
            '4078' : 'Unable to rate (4078)',
            '4079' : 'Unable to rate (4079)',
            '407A' : 'Unable to rate (407A)',
            '407B' : 'A/R exceeds limit',
            '407C' : 'Invalid Sunday delivery flag',
            '407D' : 'Invalid Priority Alert flag',
            '407E' : 'Account number is non-numeric',
            '4080' : 'COD shipments are not allowed',
            '4081' : 'Inaccessible dangerous goods are not allowed',
            '4082' : 'Sunday delivery is not allowed',
            '4083' : 'Saturday delivery freight is not allowed',
            '4084' : 'Saturday pickup freight is not allowed',
            '4085' : 'Saturday delivery is not allowed',
            '4086' : 'Saturday hold at location not allowed',
            '4087' : 'Saturday pickup requires Saturday ship date',
            '4088' : 'Saturday pickup not allowed for service',
            '4089' : 'Sunday delivery not available',
            '408A' : 'Sunday hold at location not available',
            '408B' : 'Sunday delivery not allowed for Hawaii origin',
            '408C' : 'COD not allowed with service',
            '408D' : 'COD not allowed with bill recipient',
            '408E' : 'COD not allowed with bill 3 rd party',
            '4090' : 'Credit card authorization failed.',
            '4091' : 'Invalid or missing Variable Handling Charge.',
            '4201' : 'Internal FedEx international rate error (4201)',
            '4202' : 'Internal FedEx international rate error (4202)',
            '4203' : 'Account inactive',
            '4204' : 'No location for origin',
            '4205' : 'No currency conversion available',
            '4206' : 'Negative invoice amount',
            '4207' : 'Zero invoice amount',
            '4208' : 'Invalid transaction code',
            '4209' : 'Invalid service code',
            '420A' : 'Invalid billing currency',
            '424E' : 'Invalid origin station.',
            '425F' : 'Invalid shipper account.',
            '4263' : 'Invalid shipper account, unable to rate.',
            '4264' : 'Invalid bill shipper account.',
            '426A' : 'Invalid carriage value.',
            '426B' : 'Invalid carriage currency.',
            '426E' : 'Invalid weight type.',
            '42AB' : 'Invalid origin country or zip.',
            '42AC' : 'Invalid destination country or zip.',
            '42AD' : 'Invalid destination state.',
            '42AE' : 'Invalid origin state.',
            '4253' : 'Internal FedEx rate error (4253)',
            '4300' : 'Internal FedEx rate error (4300)',
            '4303' : 'Internal FedEx rate error (4303)500157RPS: Reference Number not found with FedEx Ground.500139RPS: No record of this package was found.',
            '5010' : 'No Sender account number',
            '5011' : 'No meter number',
            '5012' : 'Account number not in database',
            '5013' : 'Meter number not in database',
            '5014' : 'Invalid package count',
            '5015' : 'Invalid package weight',
            '5016' : 'No contact name',
            '5017' : 'Invalid company close time',
            '5018' : 'Unable to pickup before close time',
            '5019' : 'Too late for pickup',
            '5020' : 'No pickup service today',
            '5021' : 'No pickup on Sunday or holiday',
            '5022' : 'Internal FedEx dispatch error (5022)',
            '5023' : 'Invalid transaction number',
            '5024' : 'Invalid address',
            '5025' : 'Invalid city',
            '5026' : 'Invalid state',
            '5027' : 'Invalid postal code',
            '5028' : 'Invalid ready time',
            '5029' : 'Invalid country code',
            '5030' : 'Invalid postal code',
            '5031' : 'Recipient postal code is invalid',
            '5032' : 'Recipient postal code is a P.O. box not served',
            '5033' : 'Recipient postal code is a served P.O. box',
            '5034' : 'No Service for recipient postal code',
            '5035' : 'Service is not available to this location',
            '5036' : 'Too late for pickup service -- Drop off available',
            '5037' : 'Constant surveillance service not allow to this location',
            '5038' : 'Dangerous goods not allow to this location',
            '5039' : 'Dry ice not allow to this location',
            '5040' : 'No hold at location service to this location',
            '5041' : 'Sender postal code is a P.O. box',
            '5042' : 'Invalid pickup time',
            '5043' : 'Recipient postal code is out of delivery area',
            '5044' : 'Ready time after cutoff time',
            '5045' : 'Cash-only account',
            '5046' : 'Missing authentication flag',
            '5047' : 'Invalid confirmation number',
            '5048' : 'Invalid location ID',
            '5049' : 'No dispatch found for this account',
            '504A' : 'Invalid customer name',
            '504B' : 'Invalid phone number',
            '504C' : 'Previous day pickup. Cannot cancel or update.',
            '504D' : 'Pickup already made. Cannot cancel or update.',
            '504E' : 'General update error (504E)',
            '504F' : 'Pickup was cancelled. Cannot update.',
            '5050' : 'Pickup is over 2 days old. Cannot update.',
            '5051' : 'Remote location. Call FedEx.',
            '5052' : 'Internal dispatch error (5052)',
            '5053' : 'Internal dispatch error (5053)',
            '5054' : 'Dispatch already exists',
            '5055' : 'Intra-country service not available',
            '5056' : 'Cannot schedule pickup beyond 1 working day',
            '5057' : 'Invalid date',
            '5058' : 'Past Express Freighter cutoff. Use International Priority Service.',
            '5059' : 'Cannot schedule dispatch',
            '505A' : 'State or postal code change not allowed',
            '505B' : 'Must reschedule to change pickup day',
            '505C' : 'Invalid package location',
            '505D' : 'Carrier code not specified',
            '505E' : 'No remarks',
            '505F' : 'Invalid dim width.',
            '5060' : 'Invalid dim length.',
            '5061' : 'Invalid dim height.',
            '5062' : 'Invalid service type',
            '5063' : 'Invalid field LTD.',
            '5064' : 'Invalid truck size.',
            '5065' : 'Invalid booking number.',
            '5066' : 'Invalid girth.',
            '5067' : 'Invalid department, floor, or suite code.',
            '5068' : 'Invalid weight.',
            '5501' : 'Fax has been queued.',
            '5502' : 'No signature was returned',
            '5504' : 'General EDR error.',
            '5506' : 'Duplicate tracking number.',
            '5508' : 'Invalid tracking number.',
            '5510' : 'Tracking number was not found.',
            '5512' : 'Ship date is required.',
            '5513' : 'Invalid ship date.',
            '5514' : 'Signature Proof of Delivery letters for FedEx shipments are available for only 18 months.',
            '5516' : 'Outside SPOD delivery area',
            '5518' : 'Invalid request type. Request type for SPOD must be L (letter) or F (fax).',
            '5520' : 'Sender phone number is required.',
            '5522' : 'Sender name is required.',
            '5524' : 'The recipient fax number is required for SPOD fax option.',
            '5525' : 'The recipient mail address line 1 is required for Ground SPOD fax option.',
            '5526' : 'The recipient fax name is required for SPOD fax option.',
            '5527' : 'The recipient mail company name is required for Ground SPOD fax option.',
            '5528' : 'The recipient fax number must be 10 digits in length.',
            '5530' : 'Invalid language code. Language code for SPOD must be EN (English) or FR (French).',
            '5532' : 'Ship date is a future date.',
            '5534' : 'Internal SPOD server error (Ground track failed).',
            '5536' : 'Invalid recipient phone number.',
            '5538' : 'Invalid sender phone number.',
            '5540' : 'Invalid recipient fax number.',
            '5542' : 'Invalid SPOD letter format.',
            '5544' : 'The recipient mail city is required for Ground SPOD fax option.',
            '5546' : 'The recipient mail state is required for Ground SPOD fax option.',
            '5548' : 'The recipient mail postal code is required for Ground SPOD fax option.',
            '5550' : 'The recipient mail country code is required for Ground SPOD fax option.',
            '5552' : 'Sender phone number must be 10 digits in length.',
            '5554' : 'Signature is unavailable on-line. Please use Fax request option, http://www.fedex.com/, orcontain Customer Service.',
            '5556' : 'Invalid fax recipient mail country code.',
            '5558' : 'Invalid recipient country code.',
            '5600' : 'Internal SPOD server error (no SPOD factory found).',
            '5602' : 'Internal SPOD server error (SPOD invalid domain).',
            '5604' : 'Internal SPOD server error (SPOD CORBA exception).',
            '5606' : 'Internal SPOD server error (SPOD exception).',
            '5608' : 'No Ground signature image was returned.',
            '5610' : 'Signature is unavailable on-line. Please use Fax request option, http://www.fedex.com/, orcontact Customer Service.',
            '5612' : 'Internal SPOD server error (Ground Track Error).',
            '5650' : 'Internal SPOD server error (Ground SPOD write error).',
            '5652' : 'Internal SPOD server error (Ground SPOD read error).',
            '5654' : 'Internal SPOD server error (Ground SPOD connect error).',
            '5656' : 'Internal SPOD server error (Ground SPOD time-out).',
            '6000' : 'Internal FedEx URSA error (6000)',
            '6001' : 'Internal FedEx URSA error (6001)',
            '6002' : 'Internal FedEx URSA error (6002)',
            '6003' : 'Internal FedEx URSA error (6003)',
            '6004' : 'Internal FedEx URSA error (6004)',
            '6005' : 'Internal FedEx URSA error (6005)',
            '6006' : 'Internal FedEx URSA error (6006)',
            '6007' : 'Internal FedEx URSA error (6007)',
            '6008' : 'Internal FedEx URSA error (6008)',
            '6009' : 'Postal code is a Post Office Box',
            '6010' : 'Internal FedEx URSA error (6010)',
            '6011' : 'Postal code not served',
            '6012' : 'Hold at Location not valid',
            '6013' : 'Saturday delivery not available',
            '6014' : 'Standard Overnight Service not available',
            '6015' : 'Invalid postal code',
            '6016' : 'Must be A1 commit for First Overnight service',
            '6017' : 'Accessible Dangerous Goods not allowed',
            '6018' : 'Inaccessible Dangerous Goods not allowed',
            '6019' : 'Dry Ice not allowed',
            '6020' : 'Invalid Express saver Economy 2 Day handling',
            '6021' : 'Express saver Economy 2 Day handling not set',
            '6022' : 'First Overnight later delivery not available to this location',
            '6023' : 'Government Overnight Service not available',
            '6024' : 'Postal code/city mismatch',
            '6025' : 'Invalid format',
            '6026' : 'Postal not found',
            '6027' : 'Invalid postal code format.',
            '6028' : 'Country not found in URSA',
            '6029' : 'Invalid FedEx URSA error (6029)',
            '6030' : 'Invalid FedEx URSA error (6030)',
            '6031' : 'Service was not found',
            '6032' : 'Handling code was not found',
            '6033' : 'Location was not found',
            '6034' : 'Service not allowed',
            '6035' : 'Too many counties with postal code for output',
            '6037' : 'Invalid FedEx URSA error (6037)',
            '6038' : 'Invalid ship date',
            '6039' : 'Invalid FedEx URSA error (6038)',
            '6040' : 'Invalid FedEx URSA error (6040)',
            '6041' : 'ASTRA location number not found',
            '6042' : 'Invalid FedEx URSA error (6042)',
            '6043' : 'Invalid FedEx URSA error (6043)',
            '6044' : 'Invalid FedEx URSA error (6044)',
            '6045' : 'Origin or Destination postal code not available for ZIP2ZIP',
            '6046' : 'URSA table status is expired',
            '6047' : 'URSA table status is early',
            '6048' : 'Desired country is not IES served',
            '6049' : 'Country in != country ou',
            '6050' : 'Requested state did not match state found in URSA lookup',
            '6051' : 'Sunday delivery not available',
            '6052' : 'Recipient state and URSA state are not the same',
            '6053' : 'Origin not allowed for Extra Hours service',
            '6054' : 'Destination not allowed for Extra Hours service',
            '6055' : 'Dangerous goods not allowed',
            '6056' : 'Dry ice not allowed',
            '7001' : 'Customer not activated for international shipping',
            '7002' : 'Rates not available',
            '7003' : 'Internal FedEx international ship error (7003)',
            '7004' : 'Unknown transaction type',
            '7005' : 'Internal FedEx international ship error (7005)',
            '7006' : 'Internal FedEx international ship error (7006)',
            '7007' : 'Internal FedEx international ship error (7007)',
            '7008' : 'Internal FedEx international ship error (7008)',
            '7009' : 'Internal FedEx international ship error (7009)',
            '7010' : 'Invalid country code',
            '7011' : 'Declared value exceeds amount allowed for this country',
            '7012' : 'Failed to obtain Master weight for multi-piece shipment calculations',
            '7013' : 'Failed to obtain multi-piece shipment weight for multi-piece shipment calculations',
            '7014' : 'Invalid package weight.',
            '7015' : 'Document shipping not allowed for this country',
            '7016' : 'Non-document shipping not allowed for this country',
            '7017' : 'Saturday delivery not allowed for this country',
            '7018' : 'International shipping has been turned off',
            '7019' : 'Internal FedEx international ship error (7019)',
            '7020' : 'Internal FedEx international ship error (7020)',
            '7021' : 'Internal FedEx international ship error (7021)',
            '7022' : 'Failed to insert record into database',
            '7023' : 'Failed to insert multi-piece shipment record into database',
            '7024' : 'Failed to read multi-piece shipment record from database',
            '7025' : 'Failed to read Customer database',
            '7026' : 'Failed to delete record from database',
            '7027' : 'Failed to update Master record in database',
            '7028' : 'Internal FedEx international ship error (7028)',
            '7029' : 'Internal FedEx international ship error (7029)',
            '7030' : 'Internal FedEx international ship error (7030)',
            '7031' : 'Internal FedEx international ship error (7031)',
            '7032' : 'Internal FedEx international ship error (7032)',
            '7033' : 'Internal FedEx international ship error (7033)',
            '7034' : 'Internal FedEx international ship error (7034)',
            '7035' : 'IES not available for this country',
            '7036' : 'Ship date not within 10 days of current date',
            '7037' : 'Sunday shipment date not allowed',
            '7038' : 'Invalid shipment date',
            '7039' : 'Intra country not allowed for this country',
            '7040' : 'Invalid service type',
            '7041' : 'Saturday delivery not allowed with service',
            '7042' : 'IFO not available for this country',
            '7043' : 'Saturday pickup not allowed with service',
            '7044' : 'Hold at location not allowed with service',
            '7045' : 'Service not available to this postal code',
            '7046' : 'Only bill sender allowed for this service',
            '7047' : 'Internal FedEx international ship error (7047)',
            '7048' : 'Internal FedEx international ship error (7048)',
            '7049' : 'Invalid currency type',
            '7050' : 'Taiwan 3 rd party error',
            '7051' : 'Shipment origin or destination must be in Taiwan',
            '7052' : 'Invalid weight unit of measurement or dim unit of measurement',
            '7053' : 'Internal FedEx international ship error (7053)',
            '7054' : 'Internal FedEx international ship error (7054)',
            '7055' : 'Total transaction number is greater than 10',
            '7056' : 'Length or girth exceeds the maximum limit',
            '7057' : 'IEF not available for this country',
            '7058' : 'IPF not available for this country',
            '7059' : 'Weight exceeds amount allowed for this country',
            '7060' : 'Invalid payment type',
            '7062' : 'Not allowed to ship commercial invoice to this country',
            '7063' : 'Internal FedEx international ship error (7063)',
            '7064' : 'Internal FedEx international ship error (7064)',
            '7065' : 'Integer number is necessary in this field',
            '7066' : 'Internal FedEx international ship error (7066)',
            '7067' : 'Invalid duties and taxes payment type',
            '7068' : 'Invalid credit card number',
            '7069' : 'Invalid credit card type',
            '7070' : 'Invalid credit card expiration date',
            '7071' : 'Invalid label type / Returns are not available with legacy label type',
            '7072' : 'Invalid dim unit of measure',
            '7073' : 'Dry ice weight error',
            '7074' : 'Inaccessible dangerous goods not allowed',
            '7075' : 'Accessible dangerous goods not allowed',
            '7076' : 'Alcohol not allowed',
            '7077' : 'Invalid package count',
            '7078' : 'Pick up service not allowed for Extra Hours from US',
            '7079' : 'Incomplete commodity description: CD, DVD, disks, Tapes, & Videos are too vague forCustoms.Please include type and title; Videotapes must include duration and subject matter.',
            '7080' : 'Incomplete commodity description: Docs, PIB, Commercial Invoice, NAFTA, & See Attachedare too vague for Customs. Please specify the commodity by name, type or use.',
            '7081' : 'Incomplete commodity description: Pharmaceuticals such as Medicine and Drug must beidentified as prescriptions or non-prescription and type.',
            '7082' : 'Incomplete commodity description: Clothes or Textiles are too vague for Customs.',
            '7083' : 'Incomplete commodity description: Dangerous Goods and Hazardous Materials must beidentified by the proper shipping name.',
            '7084' : 'Incomplete commodity description: Printed Material, Training Material, Gifts, PromotionalItems, & Personal Items are too vague for Customs.',
            '7085' : 'Incomplete commodity description: Samples and Parts are too vague for customs. Pleaseinclude type, name and use for these items.',
            '7086' : 'Incomplete commodity description: A commodity description is required for all internationalshipments.',
            '7091' : 'Selected Terms of Sale is invalid.',
            '7092' : 'Freight and Insurance amounts are not required for Terms of Sale FCA and EXW',
            '7093' : 'Freight and Insurance amounts are required with Terms of Sale: CIP, DDP, & DDU.',
            '7094' : 'Freight charges are required, Insurance amount not allowed for Terms of Sale CPT.',
            '7095' : 'The freight and insurance amounts must be greater than zero for indicated Terms of Sale.',
            '7096' : 'Incomplete commodity description: Please include a complete description.',
            '7097' : 'An Export Identification Number is required.',
            '7098' : 'The shipment XTN is not required when FedEx is filing.',
            '7099' : 'U.S. Export requires a valid AES Filing Status.',
            '7099' : 'No tracking number.',
            '7100' : 'Please enter a valid License Exception Symbol for this commodity.',
            '7101' : 'Only one of the following can be selected for a commodity: Export License, ITAR Exemption,or License Exception symbol.',
            '7102' : 'A valid expiration date is required for all shipments moving under an export license.',
            '7103' : 'A valid Export Cargo Classification Number (ECCN) is required with an export license orsymbol.',
            '7104' : 'A valid Harmonized code is required for all commodities within the shipment when SED isrequired.',
            '7105' : 'A valid Mexico State code is required for shipments moving under SED.',
            '7106' : 'This shipment requires SED. Please Enter the XTN number or indicate AES/FTSR Exemption.',
            '7107' : 'U.S. Customs requires all shippers to identify if a forwarding Agent is involved.',
            '7108' : 'Parties to Transaction not specified. Related parties must have minimum 10% ownership.',
            '7109' : 'Please indicate if this shipment is a Routed Export Transaction.',
            '7110' : 'U.S. Principal Party information is needed or select routed Export Transaction.',
            '7111' : 'U.S. Principal Party of Interest must reside in the United States.',
            '7112' : 'Please enter a valid export license with format ANNNNNN.',
            '7113' : 'Shipper filing AES, not allowed to enter: Export License, ITAR Exemption, License ExceptionSymbol, ECCN or Export License Expire Date.',
            '7114' : "Please provide complete U.S. Principal Party of Interest data or use shipper's information.",
            '7115' : 'If SED does not apply or Forwarding Agent is No. U.S. PPI data should not be populated.',
            '7116' : 'U.S. Principal Party information is not required for Shipper Filed AES',
            '7117' : 'AES/FTSR Exemption is not allowed if Export License, ITAR Exemptions or LicenseExemption symbol.',
            '7118' : 'A valid Customs Value must be entered on all international shipments.',
            '7119' : 'Broker country must be the same as the recipient country.',
            '7120' : 'Saturday delivery not allowed with Broker Select.',
            '7121' : 'Broker selected but required broker data fields are missing.',
            '7122' : 'Broker Select option not allowed for selected service type.',
            '7124' : 'Partial Intermediate Consignee details.',
            '7125' : 'Invalid customs value.',
            '7126' : 'Carriage value exceeds customs value.',
            '7128' : 'Emerge destination must be US or PR.',
            '7129' : 'Emerge Shipment ID is invalid.',
            '7130' : 'Emerge Merchant ID is invalid.',
            '7131' : 'The selected Service type is not allowed with Emerge.',
            '7132' : 'Saturday pickup is not allowed with Emerge.',
            '7133' : 'Emerge not allowed for service IPF and IEF for H3 delivery areas.',
            '7134' : 'Broker Select Option (BSO) not allowed with Emerge.',
            '7135' : 'Dry Ice shipment not allowed with Emerge.',
            '7136' : 'Signature release is not allowed with Emerge.',
            '7143' : 'Freight Services are not allowed to Hawaii.',
            '7144' : 'IPFS/IEFS shipment does not meet minimum weight or dimensional restrictions.',
            '7145' : 'Booking Number required for IPFS/IEFS. Call FedEx Freight Customer Service 1-800-3320807 to obtain booking number.',
            '7146' : 'Invalid skid count.',
            '7147' : "Shipper's Load and Count (SLAC) required for IPFS/IEFS service: must be numeric value 1 to99,999.",
            '7148' : 'IPFS/IEFS limits drop off types to values 1, 2, and 5.',
            '7149' : 'Selected Service requires Dim length, width, and height.',
            '7150' : 'Hold at location phone is required when HAL is selected.',
            '7163' : 'Invalid package sequence or package sequence exceeds package count',
            '8001' : 'Account number field is empty',
            '8002' : 'Invalid account number',
            '8003' : 'Internal FedEx Registration error (8003)',
            '8004' : 'Customer is cash only',
            '8005' : 'Internal FedEx Registration error (8005)',
            '8006' : 'Invalid name',
            '8007' : 'Invalid phone',
            '8008' : 'Invalid fax',
            '8009' : 'Meter number exists',
            '800A' : 'Internal FedEx Registration error (800A)',
            '800B' : 'Internal FedEx Registration error (800B)',
            '800C' : 'Unknown or invalid transaction',
            '800D' : 'Internal FedEx Registration error (800D)',
            '800E' : 'Account and meter do not match',
            '800F' : 'Internal FedEx Registration error (800F)',
            '8010' : 'Invalid device type',
            '8011' : 'Invalid meter number',
            '8012' : 'Sender name and/or company required',
            '8013' : 'Sender address line 1 required',
            '8014' : 'Sender city required',
            '8015' : 'Sender state required',
            '8016' : 'Recipient postal code required',
            '8017' : 'Invalid recipient country',
            '8018' : 'Sender phone number required',
            '8019' : 'Invalid phone number. Non-numeric.',
            '801A' : 'Invalid phone number. Incorrect length',
            '801B' : 'Invalid payer account number. Non-numeric in string',
            '801C' : 'Product version no longer supported or not defined',
            '801D' : 'Account number required',
            '801E' : 'Internal FedEx Registration error (801E)',
            '801F' : 'Postal code and state/province mismatch',
            '8020' : 'Invalid postal code',
            '8021' : 'Could not obtain access numbers from phone service (8021)',
            '8022' : 'Could not obtain access numbers from phone service (8022)',
            '8023' : 'Internal FedEx Registration error (8023)',
            '8024' : 'Invalid account number',
            '8025' : 'Invalid account number. Length exceeds 9 digits.',
            '8026' : 'Internal FedEx Registration error (8026)',
            '8027' : 'State or province abbreviation must be two characters',
            '8028' : 'Country abbreviation must be two characters',
            '8029' : 'Meter Number is required',
            '8030' : 'Cycle number is required',
            '803A' : 'Invalid cycle number. Non-numeric.',
            '803B' : 'Invalid cycle number',
            '803C' : 'Internal FedEx Registration error (803C)',
            '803D' : 'Internal FedEx Registration error (803D)',
            '803E' : 'Recipient name and/or company required',
            '803F' : 'Recipient address line 1 required',
            '8040' : 'Recipient city required',
            '8041' : 'Recipient state required',
            '8042' : 'Invalid weight unit of measurement',
            '8043' : 'Hold at location requires hold address',
            '8044' : 'Hold at location requires hold city',
            '8045' : 'Hold at location requires hold state',
            '8046' : 'Invalid label printer type',
            '8047' : 'Invalid label media type',
            '8048' : 'Invalid combination of label type, label printer type, and label media type',
            '8049' : 'Invalid packaging/service combination',
            '804A' : 'Invalid dry ice weight. Must contain one explicit decimal place.',
            '804B' : 'Invalid freight charge. Must contain two explicit decimal places.',
            '804C' : 'Invalid insurance charge. Must contain two explicit decimal places.',
            '804D' : 'Invalid tax misc. charge. Must contain two explicit decimal places.',
            '804E' : 'Shipment request failed due to label creation error on FedEx server.',
            '804F' : 'Shipment Contents Records Total must be a numeric value between 0 and 50.',
            '8050' : 'Invalid declared value. Must contain two explicit decimal places.',
            '8051' : 'Invalid weight. Must be greater than zero and contain one explicit decimal place.',
            '8052' : 'Invalid dimension unit of measurement.',
            '8053' : 'Invalid package sequence number.',
            '8054' : 'Package sequence is greater than package count.',
            '8055' : 'Invalid COD amount. Must contain two explicit decimal places.',
            '8056' : 'Invalid dry ice weight',
            '8057' : 'Invalid dry ice weight. Must contain one explicit decimal place.',
            '8058' : 'Invalid carrier code',
            '8059' : 'COD master tracking number required',
            '805A' : 'No ground account number',
            '805C' : 'This FedEx Ground account can only be used when dropping off packages. To find thenearest drop off location, please call FedEx Customer Service at 1-800-Go-FedEx or go tofedex.com and click on Locations.',
            '8060' : 'COD return tracking number required',
            '8061' : 'COD return address 1 required',
            '8062' : 'COD return postal code must be 5 or 9 digits',
            '8063' : 'COD return phone number must be 10 digits',
            '8064' : 'Dispatch number required',
            '8065' : 'Dispatch location required',
            '8066' : 'Unit quantity must be numeric',
            '8068' : 'Invalid commodity unit value. Must contain six explicit decimal places.',
            '8069' : 'Invalid commodity customs value 1. Must contain six explicit decimal places.',
            '806A' : 'Invalid commodity customs value 2. Must contain six explicit decimal places.',
            '806B' : 'Invalid commodity customs value 3. Must contain six explicit decimal places.',
            '806C' : 'Invalid commodity customs value 4. Must contain six explicit decimal places.',
            '806D' : 'Carriage value cannot exceed customs value.',
            '8070' : 'Invalid unit quantity',
            '8071' : 'Invalid commodity unit of measure',
            '8072' : 'Invalid commodity weight. Must contain one explicit decimal place.',
            '8073' : 'Commodity unit value 1 must be numeric',
            '8074' : 'Commodity unit value 2 must be numeric',
            '8075' : 'Commodity unit value 3 must be numeric',
            '8076' : 'Commodity unit value 4 must be numeric',
            '8077' : 'Commodity unit value 1 required',
            '8079' : 'Invalid total customs value. Must contain two explicit decimal places.',
            '807A' : 'Commodity unit value 2 required',
            '807B' : 'Commodity unit value 3 required',
            '807C' : 'Commodity unit value 4 required',
            '8080' : 'Total customs value must be numeric',
            '8081' : 'Invalid carriage value. Must contain two explicit decimal places.',
            '8082' : 'Invalid recipient state',
            '8083' : 'Duties and taxes payment type required',
            '8084' : 'Invalid duties and taxes payment type',
            '8085' : 'Duties and taxes payer account number required',
            '8086' : 'Invalid duties and taxes payer account number. Contains non-numeric character.',
            '8087' : 'Invalid duties and taxes payer account number. Length exceeds 9 digits.',
            '8088' : 'Importer name required for commercial invoice',
            '8089' : 'Importer address 1 required for commercial invoice',
            '8090' : 'Importer city required for commercial invoice',
            '8091' : 'Importer phone number required for commercial invoice',
            '8092' : 'Importer country required for commercial invoice',
            '8093' : 'Internal FedEx routing error (8093)',
            '8094' : 'Multi-piece shipment requires master trknum and master form id fields.',
            '8095' : 'Invalid total shipment weight. Must contain one explicit decimal place.',
            '8096' : 'Multi-piece shipment not allowed for requested package type.',
            '8097' : 'Accumulated commodity weight(s) cannot exceed package weight for single piece or totalshipment weight for MPS.',
            '8098' : 'Invalid Alcohol Volume. Must contain three explicit decimal places.',
            '8099' : 'Invalid Alcohol Volume. Length exceeds 6 digits.',
            '809E' : 'Invalid Alcohol Packages (number of units). Length exceeds 3 digits.',
            '8100' : 'Unable to communicate with the mainframe',
            '8300' : 'Internal consistency error -- bad transaction passed to CheckLength.',
            '8301' : 'Receive data underflow.',
            '8302' : 'Receive data overflow.',
            '8500' : 'Tracking number is required for SPOD.',
            '8501' : 'Request type is required for SPOD.',
            '8502' : 'Letter format for SPOD must be 1 (PNG format).',
            '8503' : 'The recipient fax number is non-numeric',
            '8504' : 'The recipient fax mail country must be US or CA.',
            '8505' : 'The recipient country must be US or CA.',
            '8506' : 'Signature Proof of Delivery is currently unavailable.',
            '850B' : "Shipper's Load and Count (SLAC) cannot be less than total package count.",
            '850D' : 'Invalid Booking Number for IPFS/IEFS service: must be numeric and 8 to 12 digits in length.',
            '850F' : 'Return shipment indicator not allowed with FedEx Express shipments/Invalid Return ShipmentIndicator.',
            '8510' : 'Dangerous Goods is invalid for value found in Return Shipment Indicator',
            '8511' : 'COD is invalid for value found in Return Shipment Indicator',
            '8512' : 'Variable Handling Type is invalid for value found in Return Shipment Indicator',
            '8516' : 'Print Return Label feature is not available with future day shipping',
            '8517' : 'URL Email Address cannot exceed 120 characters.',
            '8518' : 'URL Email Address cannot exceed 200 characters.',
            '8519' : 'Item Description must contain between 1 and 80 characters',
            '8520' : 'Shipment Content Records Total cannot be greater than zero for the Returns Type indicated.',
            '8601' : 'Unknown transaction.',
            '8602' : 'Insert failure.',
            '8603' : 'Tracking number is blank.',
            '8604' : 'Tracking number not found in the master database.',
            '8605' : 'Tracking number is not a master tracking number.',
            '8995' : 'Meter logon failed',
            '8996' : 'Your billing account number cannot be validated at this time.',
            '8997' : 'International ship transaction failed.',
            '8999' : 'An intermediate transaction failed (Service Availability)',
            '980B' : 'Invalid Value for Manifest Only Flag',
            '9900' : 'Internal FedEx Ground Server error (9900).',
            '9904' : 'Invalid broker country code.',
            '9911' : 'No such attribute.',
            '9912' : 'Attribute or value exists.',
            '9913' : 'Record not found.',
            '9914' : 'Entry already exists.',
            '9916' : 'Invalid or inadequate data for request.',
            '9917' : 'Required data missing',
            '9918' : 'Invalid update mode.',
            '9919' : 'Invalid search key.',
            '9920' : 'Invalid search condition.',
            '9922' : 'Unknown database host.',
            '9923' : 'Invalid or inadequate data for request.',
            '9924' : 'Invalid or inadequate data for request.',
            '9925' : 'Value cannot be deleted.',
            '9929' : 'Username/Password required',
            '9930' : 'Zip/postal code does not match country and state/province',
            '9940' : 'No scan record for this shipment has been received by our system yet. Please try again lateror contact Customer Service at 1.800.Go.FedEx.',
            '9980' : 'Internal FedEx communication error to Tracking Server',
            '9981' : 'Invalid search type code for carrier code FDXG, orInvalid search type code for carrier code FDXE.',
            '9982' : 'Invalid or missing search value.',
            '9983' : 'Invalid tracking number for carrier code.',
            '9984' : 'Invalid date range plus End date is before Begin date, Begin date is beyond 90 days, End date is beyond current date, Begin to end date is greater than 60 days, Begin date is not formatted, End date without begin date, Begin date without end date, Begin date is not formatted CCYYMMDD, End date is not formatted CCYYMMDD.',
            '9985' : 'Paging token is not properly formatted.',
            '9997' : 'Dispatch transactions not allowed at this time',
            '9998' : 'Dispatch transactions not allowed at this time',
            '9999' : 'Signature release not allowed for COD',
            'E502' : 'Invalid Carrier Code',
            'E503' : 'Invalid Service Type',
            'E504' : 'Invalid Packaging Type',
            'E505' : 'Invalid Service Type',
            'E517' : 'Rate Available services function is not available for international Express.',
            'E523' : 'Invalid Weight',
            'E524' : 'Invalid dimensions',
            'E525' : 'The DeclaredValue/CurrencyCode is not equal to the Declared Value of the Origin Country.',
            'E526' : 'Service Availability is down or transaction has timed out. Try request again later.',
            'F001' : 'Invalid State code',
            'F002' : 'Destination country required',
            'F003' : 'Only International can be processed',
            'F004' : 'Dimension length must be numeric',
            'F005' : 'Dimension width must be numeric',
            'F006' : 'Dimension height must be numeric',
            'F007' : 'Declared value must be numeric',
            'F008' : 'Invalid package count',
            'F009' : 'Package count must be numeric',
            'F00A' : 'Invalid country',
            'F00B' : 'Invalid recipient country',
            'F00C' : 'Invalid service type',
            'F00D' : 'Dimension width is greater than 99',
            'F00E' : 'Invalid package weight',
            'F00F' : 'Package weight must be numeric',
            'F010' : 'Invalid recipient postal code',
            'F011' : 'Date must be numeric',
            'F012' : 'Invalid date',
            'F013' : 'Date cannot be less than current date',
            'F014' : 'Date cannot be increased more than 7 days from current date',
            'F015' : 'Invalid dangerous goods class',
            'F016' : 'Cargo Aircraft not allowed with this service',
            'F017' : 'Invalid dangerous goods class',
            'F018' : 'Invalid tracking number',
            'F019' : 'Invalid tracking number',
            'F01A' : 'Invalid tracking number',
            'F01B' : 'Invalid tracking number',
            'F01C' : 'Recipient postal code and state/province do not match.',
            'F01D' : 'Invalid sender postal code',
            'F01E' : 'Internal FedEx Server error (F01E)',
            'F01F' : 'Weight type must be LBS (pounds) or KGS (kilograms)',
            'F019' : 'Invalid tracking number.',
            'F020' : 'Internal FedEx Server error (F020)',
            'F021' : 'Invalid currency type',
            'F022' : 'Package count 1 is empty',
            'F023' : 'Package count 2 is empty',
            'F024' : 'Package count 3 is empty',
            'F025' : 'Package count 4 is empty',
            'F026' : 'Package count 1 must be numeric',
            'F027' : 'Package count 2 must be numeric',
            'F028' : 'Package count 3 must be numeric',
            'F029' : 'Package count 4 must be numeric',
            'F02A' : 'Commodity weight 1 required',
            'F02B' : 'Commodity weight 2 required',
            'F02C' : 'Commodity weight 3 required',
            'F02D' : 'Commodity weight 4 required',
            'F02E' : 'Commodity weight 1 must be numeric',
            'F02F' : 'Commodity weight 2 must be numeric',
            'F030' : 'Commodity weight 3 must be numeric',
            'F031' : 'Commodity weight 4 must be numeric',
            'F032' : 'Invalid customs value 1',
            'F033' : 'Commodity customs value 1 must be numeric',
            'F034' : 'Commodity customs value 2 must be numeric',
            'F035' : 'Commodity customs value 3 must be numeric',
            'F036' : 'Commodity customs value 4 must be numeric',
            'F037' : 'Package count is greater than 999',
            'F038' : 'Package weight is zero',
            'F039' : 'Internal FedEx Server error (F039)',
            'F03A' : 'International Priority Freight Service not available to this country',
            'F03B' : 'International Priority Freight Service dimension height is greater than 70 inches',
            'F03C' : 'Invalid sender country',
            'F03D' : 'Origin country required',
            'F03E' : 'Drop off not available with this service',
            'F03F' : 'Saturday Pickup and Saturday Delivery mutually exclusive',
            'F040' : 'Inconsistent dimension and weight units. Use C (centimeters) with KG and I (inches) with LBS.',
            'F041' : 'Invalid Alcohol Type: Alcohol type is required and must be B, W, S, A or L.',
            'F042' : 'Invalid Alcohol Package Type: Alcohol package type is required and must be BL, BT, CS, CNor OT.',
            'F043' : 'Invalid Alcohol Package quantity. Alcohol package quantity is required and must be a numericvalue.',
            'F044' : 'Invalid Alcohol Volume: Alcohol volume is required and must contain a numeric value.',
            'F04A' : 'Alternate Sender Address phone is required.',
            'F04B' : 'PendingIncomplete Alternate Sender Address information',
            'F04C' : 'PendingAlternate Sender Address City required',
            'F04D' : 'PendingAlternate Sender Address state is required.',
            'F04E' : 'Invalid Alternate Sender Address postal code.',
            'F04F' : 'Sender Address country and Alternate Sender Address country must be the same.',
            'F05A' : 'Recipient country requires a postal code served by FedEx.',
            'F051' : 'Meter logon failure.',
            'F052' : 'Sender postal code state/providence do not match',
            'F057' : 'Invalid Sender postal code format for specified country',
            'F058' : 'Sender country requires a postal code served by FedEx.',
            'F059' : 'Invalid Recipient postal code format for specified country',
            'F06A' : 'Invalid Carrier code',
            'F06B' : 'Invalid number of items',
            'F06F' : 'Invalid item description',
            'F070' : 'Invalid URL email address',
            'F071' : 'Invalid shipper phone.',
            'F072' : 'Invalid service and packaging combination.',
            'F074' : 'Invalid shipper state.',
            'F075' : 'Invalid form ID.',
            'F076' : 'Invalid merchant phone number.',
            'F077' : 'XML Parser error.',
            'F078' : 'Invalid account number.',
            'F079' : 'Invalid meter number',
            'F07A' : 'Invalid payor type.',
            'F07B' : 'Invalid shipper name',
            'F07C' : 'Either shipper name or shipper company is required.',
            'F07D' : 'Invalid shipper company',
            'F07E' : 'Invalid shipper address line 1',
            'F07F' : 'Invalid shipper address line 2',
            'F080' : 'Invalid shipper city.',
            'F082' : 'Invalid Recipient name',
            'F083' : 'Invalid Recipient company',
            'F084' : 'Invalid Recipient address line 1',
            'F085' : 'Invalid Recipient address line 2',
            'F086' : 'Invalid Recipient city.',
            'F087' : 'Invalid Recipient state for specified country',
            'F088' : 'Invalid Recipient phone',
            'F089' : 'Invalid recipient email address',
            'F08F' : 'Invalid item description',
            'F090' : 'Invalid reference information',
            'F092' : 'Only weekday delivery is allowed with Ground service.',
            'F093' : 'Invalid RMA number.',
            'F094' : 'Invalid return shipment indicator',
            'F095' : 'COD is not allowed with returns.',
            'F096' : 'Alcohol is not allowed with returns.',
            'F097' : 'Emerge is not allowed with returns.',
            'F098' : 'Dangerous Goods are not allowed with returns.',
            'F09A' : 'Recipient name and/or company required.',
            'F490' : 'The FedEx RATING service is not responding.',
            'F491' : 'The FedEx ROUTING service is not responding.',
            'F492' : 'The FedEx AIRBILL service is not responding.',
            'F493' : 'The FedEx SHIP service is not responding.',
            'F494' : 'The FedEx ADMIN service is not responding.',
            'F500' : 'You must provide a billing (payer) account number',
            'F501' : 'Weight must be in whole numeric pounds',
            'F503' : 'Declared value in whole numeric dollars',
            'F510' : 'Tracking Number must be numeric',
            'F511' : 'Customs value must be numeric',
            'F512' : 'Carriage value must be numeric',
            'F513' : 'Carriage value [carriage_val] exceeds Customs value [customs_val]',
            'F520' : 'Maximum length = [len from country.txt] but length = [ len ]',
            'F521' : 'Maximum girth = [ girth from country.txt ] but girth = [ girth ]',
            'F530' : 'Unable to track package %s. Airbill not found.',
            'F54A' : '',
            'F560' : '<global error message>',
            'F570' : 'Weight [ weight ] [ lbs/kgs ] exceeds max of [ weight from country.txt ] for [ country ]',
            'F571' : 'COD Amount must be between 0 and $750',
            'F572' : 'COD Amount should not be greater $25000.00',
            'F573' : 'COD Amount must be under $25000',
            'F574' : 'Freight charges cannot be calculated',
            'F575' : 'FedEx Home Delivery not allowed with COD or ECOD',
            'F576' : 'COD not Allowed with payment type',
            'F57B' : 'Invalid destination country for COD service',
            'F57D' : 'FedEx Home Delivery not allowed with COD or ECOD.',
            'F57E' : 'If COD Add Freight Flag (186) equals 7,8 or 9 for the last package, then no other first orintermediate package can have COD Flag=\'Y\'',
            'F57F' : 'Invalid COD Add Freight option for the master package',
            'F589' : 'Variable Handling Charge not allowed with this Sender Country code.',
            'F58A' : 'Invalid or missing Variable Handling Charge.',
            'F58B' : 'Invalid Variable Handling Charge Level.',
            'F58C' : 'Sender Address country and Alternate Sender Address country must be the same',
            'F58D' : 'Invalid origin and destination pair for Commercial invoice upload.',
            'F58E' : 'Invalid or missing data elements to complete Commercial Invoice upload.',
            'F58F' : 'Invalid or missing data elements to complete Commercial Invoice upload.',
            'F590' : 'Invalid or missing data elements to complete Commercial Invoice upload.',
            'F591' : 'Invalid FTSR exemption number for this service',
            'F592' : 'Invalid or missing AES or FTSR exemption number',
            'F593' : 'Invalid or missing AES or FTSR exemption number',
            'F5A0' : 'Invalid zip code',
            'F5A1' : 'Zip Code is a PO Box or Postal Code not found',
            'F5A4' : 'Invalid Zip Code format',
            'F5A6' : 'Zip Code and cit/state mismatch',
            'F5AD' : 'Zip Code or country is not served',
            'F5BD' : 'Zip Code is missing',
            'F600' : 'Registration not allowed from selected country.',
            'F601' : 'Incomplete address for Address Verification request.',
            'F602' : 'Invalid zip code for Address Verification request.',
            'F603' : 'Authorization error while processing Address Verification request, Please contact FedEx FSMAPI hotline.',
            'F604' : 'General error processing Address Verification request.',
            'F605' : 'Multiple errors occurred while processing Address Verification request.',
            'F606' : 'Time-out error while processing Address Verification request.',
            'F607' : 'Address Verification is currently unavailable',
            'F650' : 'Invalid Address',
            'F651' : 'Invalid Origin Address',
            'F652' : 'Invalid Destination Address',
            'F653' : 'FedEx Locator unavailable -- please try again later.',
            'F654' : 'XML Parse Error in input.',
            'F655' : 'No FedEx locations were found.',
            'F656' : 'Invalid Parameters.',
            'F657' : 'Cannot search for Hold At Location and Ground.',
            'F658' : 'Cannot search for Saturday Service and Ground.',
            'F659' : 'Cannot search for Dangerous Goods and Ground.',
            'F660' : 'Cannot search for Ground and Self-Service.',
            'F661' : 'Undefined error -- please try again later.',
            'F662' : 'Parsing error in Gateway (Response).',
            'F663' : 'FedEx Locator Server Error.',
            'F664' : 'FedEx Locator Server Error (Communication Channel Error).',
            'F665' : 'FedEx Locator Server Error (Invalid XML Length).',
            'F667' : 'Unknown FedEx Locator Server Error.',
            'F800' : 'User ID must be at least three digits',
            'F801' : 'Invalid dimensional height',
            'F802' : 'Invalid dimensional width',
            'F803' : 'Invalid dimensional length',
            'F804' : 'Invalid dimensional units',
            'F805' : 'Declared value cannot exceed $25000',
            'F806' : 'Label format code must be 1, 2, or 3',
            'F807' : 'Label Type must be 1',
            'F808' : 'Label Printer Type must be 1, 2, or 3',
            'F809' : 'Label Media Type must be 1, 2, 3, 4 or 5',
            'F80A' : 'COD Flag must be Y or N',
            'F80B' : 'COD Add Freight Flag must be Y or N',
            'F80C' : 'COD Collection Type must be 1, 2, or 3',
            'F80D' : 'Home Delivery Flag must be Y or N',
            'F80E' : 'Home Type must be 1 or 2',
            'F80F' : 'Home Delivery date must be 8 numeric digits',
            'F810' : 'Residential Delivery Flag must be Y or N',
            'F811' : 'Auto POD Flag must be Y or N',
            'F812' : 'AOD Flag must be Y or N',
            'F813' : 'Hazardous Materials Flag must be Y or N',
            'F814' : 'Date required for Date Certain Delivery Service',
            'F815' : 'Telephone number required for Date Certain Delivery Service',
            'F816' : 'Telephone number required for Appointment Delivery Service',
            'F817' : 'Additional Handling Flag must be 1 or 3',
            'F818' : 'Additional Handling Amount must be between 0 and $9,999,999.99',
            'F819' : 'Additional Handling Percentage must be between 0 and 999 percent (whole numbers only)',
            'F81A' : 'When using Additional Handling Flag, either Additional Handling Amount or AdditionalHandling Percentage are required',
            'F81B' : 'Date chosen for Date Certain Delivery Service is a holiday',
            'F820' : 'Ground Packaging type must be 01',
            'F821' : 'Ground Service Type must be 90 or 92',
            'F822' : 'Home Delivery Telephone number must be 10 numeric digits',
            'F823' : 'Electronic COD Flag must be Y or N',
            'F824' : 'Pre-Delivery Alert Flag must be Y or N',
            'F825' : 'COD and Electronic COD cannot both be used at the same time',
            'F826' : 'Cash or Guaranteed funds cannot be used with Electronic COD',
            'F827' : 'Combination of special services chosen are not valid for this shipment',
            'F828' : 'Electronic COD is not allowed with Bill Third Party',
            'F829' : 'COD is not allowed with Bill Collect unless AutoPOD is also selected',
            'F82A' : 'Future Day Shipment Flag must be Y or N.',
            'F82B' : 'Recipient state/province cannot exceed 14 characters.',
            'F82C' : 'Recipient fax must be 10 digits.',
            'F82D' : 'Recipient fax cannot be greater than 15 digits.',
            'F82E' : 'Recipient pager number cannot be greater than 15 alpha-numeric characters',
            'F82F' : 'Recipient IRS EIN number cannot be greater than 15 alpha-numeric characters',
            'F830' : 'Purchase order number cannot exceed than 35 characters',
            'F831' : 'Invoice number cannot exceed 30 characters',
            'F832' : 'Customer reference number cannot exceed 30 characters',
            'F833' : 'Shipper contact name cannot exceed 35 characters',
            'F834' : 'Shipper company name cannot exceed 35 characters',
            'F835' : 'Shipper address line 1 cannot exceed 35 characters',
            'F836' : 'Shipper address line 2 cannot exceed 35 characters',
            'F837' : 'Shipper city cannot exceed 20 characters',
            'F838' : 'Shipper email address cannot exceed 80 characters',
            'F839' : 'Recipient contact name cannot exceed 35 characters',
            'F83A' : 'Recipient company name cannot exceed 35 characters',
            'F83B' : 'Recipient address line 1 cannot exceed 35 characters',
            'F83C' : 'Recipient address line 2 cannot exceed 35 characters',
            'F83D' : 'Recipient city cannot exceed 20 characters',
            'F83E' : 'Recipient email address cannot exceed 80 characters',
            'F83F' : 'Shipper contact name must be at least 3 characters',
            'F840' : 'Shipper company name must be at least 3 characters',
            'F841' : 'Shipper address line 1 must be at least 3 characters',
            'F842' : 'Shipper city must be at least 3 characters',
            'F843' : 'Recipient contact name must be at least 3 characters',
            'F844' : 'Recipient company name must be at least 3 characters',
            'F845' : 'Recipient address line 1 must be at least 3 characters',
            'F846' : 'Recipient city must be at least 3 characters',
            'F847' : 'Sender Name and/or company required',
            'F848' : 'Recipient Name and/or company required',
            'F849' : 'Invalid recipient state code for US',
            'F84A' : 'Invalid recipient province code for Canada',
            'F84B' : 'Invalid shipper state code for US',
            'F84C' : 'Invalid shipper province code for Canada',
            'F84D' : 'Shipper state/province cannot exceed 14 characters',
            'F84E' : 'Shipper fax must be 10 digits',
            'F84F' : 'Shipper fax cannot be greater than 15 digits',
            'F850' : 'Shipper pager number cannot be greater than 15 alpha-numeric characters',
            'F851' : 'Shipper IRS EIN number cannot be greater than 15 characters',
            'F852' : 'Shipper phone required',
            'F853' : 'Recipient phone required',
            'F854' : 'Ship date required (format YYYYMMDD)',
            'F855' : 'Ship time required (format HHMMSS)',
            'F856' : 'Shipper name and/or company must both be at least 3 characters in length',
            'F857' : 'Recipient name and/or company must both be at least 3 characters in length',
            'F858' : 'When using FedEx Home Delivery, either Signature Service or Appointment Delivery isrequired when using AutoPOD',
            'F859' : 'Non-standard container flag must be Y or N',
            'F85A' : 'User ID is a required field',
            'F85B' : 'User ID cannot be greater than 40 characters',
            'F85C' : 'Fatal Ground Ship Error - shipper object not instantiated',
            'F85D' : 'Fatal Ground Ship Error - recipient object not instantiated',
            'F85E' : 'Fatal Ground Ship Error - shipment object not instantiated',
            'F85F' : 'Fatal Ground Ship Error - carrier object not instantiated',
            'F860' : 'Fatal Ground Ship Error - shipper address object not instantiated',
            'F861' : 'Fatal Ground Ship Error - recipient address object not instantiated',
            'F862' : 'Fatal Ground Ship Error - single package not instantiated',
            'F863' : 'Fatal Ground Ship Error - shipment object contains no single package objects within vector',
            'F864' : 'Shipper department name cannot be greater than 10 characters',
            'F865' : 'Recipient department name cannot be greater than 10 characters',
            'F900' : 'Cannot ship from Puerto Rico',
            'F901' : 'User ID must be numeric',
            'FA01' : 'This payment type is invalid to the selected destination',
            'FA02' : 'Girth + length exceeds maximum of 165 inches for service type',
            'FB07' : 'Label Type must be 1 or 2',
            'FC01' : 'Unknown or invalid Email Label Server transaction',
            'FC05' : 'Internal Email Label Server error (FC05)',
            'FC06' : 'Internal Email Label Server error (FC06)',
            'FC07' : 'Label has already been cancelled',
            'FC08' : 'Invalid URL expiration date',
            'FC09' : 'Invalid label Request date/timestamp.',
            'FC10' : 'Internal Email Label Server error (FC10)',
            'FC11' : 'Invalid software version',
            'FC12' : 'Invalid software revision',
            'FC13' : 'Label has expired',
            'FC14' : 'Invalid contact name',
            'FC15' : 'Invalid contact phone',
            'FC17' : 'Internal Email Label Server error (FC17)',
            'FC18' : 'Email Label does not exist in that database with this tracking number.',
            'FC19' : 'Label has already been accessed by return shipper.',
            'FC1B' : 'Internal Email Label Server error (FC1B)',
            'FC1C' : 'Internal Email Label Server error (FC1C)',
            'FC1D' : 'Internal Email Label Server error (FC1D)',
            'FC1E' : 'Internal Email Label Server error (FC1E)',
            'FC20' : 'Internal Email Label Server error (FC20)',
            'FC21' : 'Internal Email Label Server error (FC21)',
            'FC22' : 'Internal Email Label Server error (FC22)',
            'FC23' : 'Internal Email Label Server error (FC23)',
            'FC24' : 'Tracking number required for cancel',
            'FC25' : 'URL expiration date cannot be more than 30 days',
            'FC26' : 'URL expiration date is required',
            'FC14' : 'Invalid contact name',
            'FCFF' : 'Internal Email Label Server error (FCFF)',
            'FE00' : 'Special Service AOD not available to selected destination',
            'FE01' : 'Special Service Auto POD not available to selected destination',
            'FE02' : 'Special Service COD not available to selected destination',
            'FE03' : 'Special Service Electronic COD not available to selected destination',
            'FE04' : 'Special Service Hazardous Materials not available to selected destination',
            'FE05' : 'Special Service Pre-Delivery Alert not available to selected destination',
            'FE06' : 'FedEx Home Delivery Special Service Date Certain not available with selected service',
            'FE07' : 'FedEx Home Delivery Special Service Delivery by Appointment not available to selecteddestination',
            'FE08' : 'FedEx Home Delivery Special Service Evening Appointment not available to selecteddestination',
            'FE09' : 'FedEx Home Delivery Special Service Signature Delivery not available to selected destination',
            'FE0A' : 'The Home Delivery service selected requires authorization before use. Please contact 1-800GO-FEDEX or your service representative for more information.',
            'FE0B' : 'The residential delivery service selected requires authorization before use. Please contact 1800-GO-FEDEX or your service representative for more information.',
            'FE0C' : 'The commercial delivery service selected requires authorization before use. Please contact 1800-GO-FEDEX or your service representative for more information',
            'FE0D' : 'Shipments for Home Delivery service must be designated as residential delivery also',
            'FE0E' : 'Delivery date for Home Delivery Date Certain service cannot be more than 16 in future',
            'FE0F' : 'Delivery date for Home Delivery Date Certain service cannot be more than 6 days in future',
            'FE10' : 'User ID not found in customer profile subscriber database',
            'FE11' : 'The Auto POD service selected requires authorization before use. Please contact 1-800-GOFEDEX or your service representative for more information.',
            'FE12' : 'The COD service selected requires authorization before use. Please contact 1-800-GOFEDEX or your service representative for more information.',
            'FE13' : 'The Electronic COD service selected requires authorization before use. Please contact 1-800GO-FEDEX or your service representative for more information.',
            'FE14' : 'The Hazardous Materials service selected requires authorization before use. Please contact1-800-GO-FEDEX or your service representative for more information.',
            'FE15' : 'Special Service Pre-Delivery Alert not available for this user ID',
            'FE16' : 'Special Service AOD not available for this user ID',
            'FE17' : 'Special Service Delivery by Appointment not available for this user ID',
            'FE18' : 'Special Service Evening Appointment not available for this user ID',
            'FE19' : 'Special Service Signature Delivery not available for this user ID',
            'FE1A' : 'Pre-Delivery Alert requires either Recipient Email ID or Fax number exclusively, but not both',
            'FE1B' : 'Either Recipient Email ID or Fax number must be used with Pre-Delivery Alert',
            'FE1C' : 'Declared value requires two explicit decimal positions (i.e. N.NN formatting)',
            'FE1D' : 'COD Amount requires two explicit decimal positions (i.e. N.NN formatting)',
            'FE1E' : 'Total Package Weight requires one explicit decimal position (i.e. N.N formatting)',
            'FE1F' : 'Additional handling charge amount requires two explicit decimal positions (i.e. N.NNformatting)',
            'FE20' : 'Optional Message for Pre-Delivery Alert cannot exceed 74 characters',
            'FE21' : 'Email Address for Pre-Delivery Alert cannot exceed 120 characters',
            'FE22' : 'Fax number for Pre-Delivery Alert must be 15 digits only',
            'FE23' : 'Fax number for Pre-Delivery Alert cannot exceed 15 digits',
            'FE24' : 'FedEx Ground Signature Service not available when shipment is not selected for FedExHome Delivery',
            'FE25' : 'FedEx Ground Date Certain Delivery Service not available when shipment is not selected forFedEx Home Delivery',
            'FE26' : 'FedEx Ground Evening Delivery Service not available when shipment is not selected forFedEx Home Delivery',
            'FE27' : 'FedEx Ground Appointment Delivery Service not available when shipment is not selected forFedEx Home Delivery',
            'FE28' : 'Cannot create shipment - maximum of 20 duplicate FedEx Ground tracking numbersencountered on FedEx Ship Server - please contact FedEx for assistance',
            'FE29' : 'COD Collection Type cannot be 2 (COD GUARANTEED FUNDS) or 3 (COD CURRENCY)when a shipment is Electronic COD',
            'FE2A' : 'Cannot create shipment - FedEx Ground tracking number already exists on FedEx ShipServer - please contact FedEx for assistance to fix problem with duplicate tracking numbers',
            'FE2B' : 'Ground Shipping is not authorized for this user id - please contact FedEx for assistance inobtaining a FedEx Ground Account number',
            'FE2C' : 'Ground Shipping is not authorized for this user id - please contact FedEx for assistance inobtaining a FedEx Express Account number',
            'FE2D' : 'The FedEx Ground Residential Delivery service cannot be used with the selected origin anddestination. Please contact 1-800-GO-FEDEX or your service representative for moreinformation.',
            'FE2E' : 'The FedEx Ground Home Delivery Service cannot be used outside the Continental U.S.Please contact 1-800-GO-FEDEX or your service representative for more information.',
            'FE2F' : 'The FedEx Ground Home Delivery Service is not available for the destination postal codechosen. Please contact 1-800-GO-FEDEX or your service representative for more information.',
            'FE30' : 'Home Delivery date is not a valid date',
            'FE31' : 'Optional Delivery Instructions for FedEx Home Delivery cannot exceed 74 characters',
            'FE32' : 'This shipment was designated as Residential Delivery but qualifies for FedEx Home Delivery.Please mark this shipment with the service \'FedEx Home Delivery\' and re-ship.',
            'FE33' : 'This currency type for this shipment does not match the origin country - please use USD forUS-originating shipments and CAD for Canada-originating shipments',
            'FF01' : 'FedEx Ground Rating Error - dynamically defined at run time',
            'FF02' : 'FedEx Ground Rating Error - Package dimensions too large',
            'FF03' : 'FedEx Ground Rating Error - Carrier Code passed not supported',
            'FF04' : 'FedEx Ground Rating Error - Service Code not available for destination area',
            'FF05' : 'FedEx Ground Rating Error - Origin Postal Code does not match Origin country',
            'FF06' : 'FedEx Ground Rating Error - Destination Postal Code does not match Destination country',
            'FF07' : 'FedEx Ground Rating Error - FedEx Home Delivery Packages may not exceed 70 lbs inweight',
            'FF08' : 'FedEx Ground Rating Error - Appointment Delivery already requires a signature."',
            'FF09' : 'FedEx Ground Rating Error - Appointment Delivery already allows consignee to select dayand time of delivery',
            'FF0A' : 'FedEx Ground Rating Error - You must select Appointment Delivery instead of Select andEvening Delivery',
            'FF0B' : 'FedEx Ground Rating Error - AutoPOD must be accompanied by an accessorial whichrequires a signature',
            'FF0C' : 'FedEx Ground Rating Error - Electronic C.O.D. can not be selected with a payment type of Bill3rd Party',
            'FF0D' : 'FedEx Ground Rating Error - Invalid weight value or invalid weight format',
            'FF0E' : 'Future Day Shipment Flag is a required field',
            'FF0F' : 'Future Day Shipment date cannot be more than 10 days in future',
            'FF10' : 'Could not establish a TCP/IP connection pool',
            'FF11' : 'Could not get an FSS CST TCP/IP connection',
            'FF12' : 'Communications failure when uploading to FSS',
            'FF13' : 'Could not login to FSS when uploading shipment',
            'FF14' : 'Communications failure attempting FSS Logon',
            'FF15' : 'FedEx Ground Ship Engine Error -- communications failure when creating shipment queue',
            'FF16' : 'FedEx Ground Ship Engine Error -- communications failure when adding shipment to queue',
            'FF17' : 'FedEx Ground Ship Engine Error -- communications failure when deleting shipment from',
            'FF30' : 'FedEx Customer Profile Error - cannot access Customer Profile Database',
            'FF31' : 'Tracking Number Service Error - could not access tracking number service',
            'FF32' : 'Tracking Number Service Error - Ground Account number does not exist',
            'FF33' : 'Tracking Number Service Error - Invalid tracking number',
            'FF34' : 'Tracking Number Service Error - Invalid tracking number length',
            'FF35' : 'Tracking Number Service Error - Invalid account number',
            'FF36' : 'Tracking Number Service Error - Invalid start number',
            'FF37' : 'Tracking Number Service Error - Invalid end number',
            'FF38' : 'Tracking Number Service Error - Invalid package number range',
            'FF39' : 'Tracking Number Service Error - unexpected tracking number service error',
            'FF3A' : 'FedEx Ground Tracking number not found in system',
            'FF3B' : 'FedEx Ground Tracking number must be 15 numeric digits in length',
            'FF3C' : 'Invalid weight for FedEx Ground Service - exceeds 150 pound limit',
            'FF3D' : 'Invalid weight for FedEx Ground Home Delivery Service - exceeds 70 pound limit',
            'FF3E' : 'Invalid weight for FedEx Ground Service to Canada - exceeds 150 pound limit',
            'FF3F' : 'Invalid weight for FedEx Ground Service to Canada - exceeds 70 pound limit',
            'FF40' : 'FedEx Account chosen for use as third-party billing for FedEx Ground Shipping is a FedExnon-revenue account number',
            'FF41' : 'Recipient contact name required when using FedEx Home Delivery Services Date Certain,Evening Delivery, or Appointment Delivery',
            'FF42' : 'Pre-Delivery Alert Error - Recipient Email ID address is not in the format ofXXXX@YYYYY.ZZZ',
            'FF43' : 'No pickup on Saturday, Sunday or holiday',
            'FF44' : 'Fax number for Pre-Delivery Alert cannot exceed 10 digits when destination is US',
            'FF45' : 'Carrier Code must be FDXG for FedEx Ground Shipments',
            'FF46' : 'FedEx Account number used for bill 3rd party is not authorized for billing ground shipments',
            'FF47' : 'Automatic subscription failed. Please subscribe before login',
            'FF48' : 'NetReturn only available with service type 92',
            'FF49' : 'NetReturn only available with pay type 1',
            'FF4A' : 'Customer Reference Number is missing',
            'FF4B' : 'Purchase Order Number is missing',
            'FF4C' : 'Invoice Number is missing',
            'FFF0' : 'FedEx Ground Parameter Validation System Failure'
        
                     }


    def _submit(self, transaction):
        h = httpslib.HTTPSConnection(self.host)
        h.putrequest('POST', '/GatewayDC')
        h.putheader('Referer', self.referer)
        h.putheader('Host', self.host)
        h.putheader('Accept', 'image/gif, image/jpeg, image/pjpeg, text/plain, text/html, */*')                               
        h.putheader('Content-type', 'image/gif')
        h.putheader('Content-length', str(len(transaction)))
        
        h.endheaders()
        
        h.send(transaction)
        
        response = h.getresponse()
        content = response.read()
        
        return content
        
    def getRate(self, order):
        """
        Build an xml document with the proper shipping information
        """
        date = self.nextBusinessDay()

        transaction = '<?xml version="1.0" encoding="UTF-8" ?>'
        transaction += '<FDXRateRequest xmlns:api="http://www.fedex.com/fsmapi" '
        transaction += 'xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
        transaction += 'xsi:noNamespaceSchemaLocation="FDXRateRequest.xsd">'
        
        transaction += '<RequestHeader>'
        transaction += '<CustomerTransactionIdentifier>sixfeetup</CustomerTransactionIdentifier>'
        transaction += '<AccountNumber>' + self.accountNumber + '</AccountNumber>'
        transaction += '<MeterNumber>' + self.meterNumber + '</MeterNumber>'
        transaction += '<CarrierCode>' + order['carrierCode'] + '</CarrierCode>'
        transaction += '</RequestHeader>'
        transaction += '<ShipDate>' + date + '</ShipDate>'
        transaction += '<DropoffType>' + order['doType'] + '</DropoffType>'
        transaction += '<Service>' + order['service'] + '</Service>'
        transaction += '<Packaging>' + order['packaging'] + '</Packaging>'
        transaction += '<WeightUnits>LBS</WeightUnits>'
        transaction += '<Weight>' + order['weight'] + '</Weight>'
        transaction += '<OriginAddress>'
        transaction += '<StateOrProvinceCode>' + order['toState'] + '</StateOrProvinceCode>'
        transaction += '<PostalCode>' + order['toZip'] + '</PostalCode>'
        transaction += '<CountryCode>' + order['toCountry'] + '</CountryCode>'
        transaction += '</OriginAddress>'
        transaction += '<DestinationAddress>'
        transaction += '<StateOrProvinceCode>' + order['fromState'] + '</StateOrProvinceCode>'
        transaction += '<PostalCode>' + order['fromZip'] + '</PostalCode>'
        transaction += '<CountryCode>' + order['fromCountry'] + '</CountryCode>'
        transaction += '</DestinationAddress>'
        transaction += '<Payment><PayorType>SENDER</PayorType></Payment>'
        transaction += '<HomeDelivery><Type>DATECERTAIN</Type></HomeDelivery>'
        transaction += '<PackageCount>' + order['packNum'] + '</PackageCount>'
        
        transaction += '</FDXRateRequest>'

        content = self._submit(transaction)
        LOG(PROJECTNAME, INFO, "This is in the FedEx Request\n\n" + str(content))
        
        end = content.find("<Error>")
        if end == -1:
            end = content.find("</NetCharge>")
            newstring = content.replace(content[end:len(content)], "")
        
            beg = newstring.find("<NetCharge>")
        
            content = newstring.replace(newstring[0:beg], "")
            content = content.replace("<NetCharge>", "")
            
        else:
            end = content.find("</Code>")
            newstring = content.replace(content[end:len(content)], "")
            
            beg = newstring.find("<Code>")
            
            content = newstring.replace(newstring[0:beg], "")
            content = content.replace("<Code>", "")
            #content = content.replace(" ", "%20")

            try:
                s = (content, self.error[content])
            except KeyError:
                s = (content, "Uknown error")
                
    
            content = s
            
        #import pdb;pdb.set_trace()
        return content
        
        
    # Write an interface in the ZMI to get the meter number
    # it should include
    #
    #
    def getMeter(self):
        transaction = { 
                'RequestHeader' :   { 'CustomerTransactionIdentifier'   : 'sixfeetup',
                                      'AccountNumber'                   : '255871609'},

                
                'Contact'       :   { 'PersonName'      :   'Kristofer Buffington',
                                      'CompanyName'     :   'Six Feet Up',
                                      'PhoneNumber'     :   '8667493338' },
                
                'Address'       :   { 'Line1'               :   '12 South Main Street',
                                      'City'                :   'Fortville',
                                      'PostalCode'          :   '46040',
                                      'StateOrProvinceCode' :   'IN',
                                      'CountryCode'         :   'US' }
                }

        header = '<FDXSubscriptionRequest xmlns:api="http://www.fedex.com/fsmapi" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"  xsi:noNamespaceSchemaLocation="FDXSubscriptionRequest.xsd">\n'
        footer = '</FDXSubscriptionRequest>\n'
        
        d = self.buildXML(transaction, header, footer)
        print d    
        reply = self._submit(d)
        
        return reply
        
    def nextBusinessDay(self):
        """
        returns the next business day in time format
        """
        d = time.localtime(time.time() + 3600*24)
        tom = date(d[0], d[1], d[2])
        x = 1
        
        while tom.weekday() > 4:
            d = time.localtime(time.time() + 3600 * (24 * x))
            tom = date(d[0], d[1], d[2])
            
            x = x+1
            
        if d[1] < 10:
            month = '0'
        
        else:
            month = ''
            
        if d[2] < 10:
            day = '0'
        
        else:
            day = ''
            
        ret = str(str(d[0]) + '-' + month + str(d[1]) + '-' + day + str(d[2]))
        
        return ret
        
        
    def buildXML(self, d, header, footer):
        """
        Builds an XML document from a dict
        """
        
        ret = '<?xml version="1.0" encoding="UTF-8" ?>'
        ret = ret + header
        
        for a in d.keys():
            ret = ret + '<'+a+'>'
            if type(d[a]) == dict:
                for j in d[a].keys():
                    ret = ret + '<'+j+'>' 
                    ret = ret + d[a][j]
                    ret = ret + '</'+j+'>'
            else:
                ret = ret + d[a]
            ret = ret + '</'+a+'>'
        
        ret = ret + footer
        
        return ret
      
